import dis
import inspect
import os
import unittest
from collections import defaultdict
from pathlib import Path
from typing import Dict, Any

import pytest
from parameterized import parameterized_class

from checkov.cloudformation.checks.resource.registry import cfn_registry
from checkov.cloudformation import cfn_utils
from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck
from checkov.cloudformation.parser import parse
from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.bridgecrew.severities import BcSeverities, Severities
from checkov.common.graph.db_connectors.networkx.networkx_db_connector import NetworkxConnector
from checkov.common.graph.db_connectors.rustworkx.rustworkx_db_connector import RustworkxConnector
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.runner_filter import RunnerFilter
from checkov.cloudformation.runner import Runner
from checkov.common.output.report import Report
from checkov.cloudformation.cfn_utils import create_definitions

RESOURCES_DIR = Path(__file__).parent / "resources"


@parameterized_class([
   {"db_connector": NetworkxConnector},
   {"db_connector": RustworkxConnector}
])
class TestRunnerValid(unittest.TestCase):

    def setUp(self) -> None:
        self.orig_checks = cfn_registry.checks

    def test_registry_has_type(self):
        self.assertEqual(cfn_registry.report_type, CheckType.CLOUDFORMATION)

    def test_record_relative_path_with_relative_dir(self):

        # test whether the record's repo_file_path is correct, relative to the CWD (with a / at the start).

        # this is just constructing the scan dir as normal
        current_dir = os.path.dirname(os.path.realpath(__file__))
        scan_dir_path = os.path.join(current_dir, "resources")

        # this is the relative path to the directory to scan (what would actually get passed to the -d arg)
        dir_rel_path = os.path.relpath(scan_dir_path).replace('\\', '/')

        runner = Runner(db_connector=self.db_connector())
        checks_allowlist = ['CKV_AWS_20']
        report = runner.run(root_folder=dir_rel_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework='cloudformation', checks=checks_allowlist))

        all_checks = report.failed_checks + report.passed_checks
        self.assertGreater(len(all_checks), 0)  # ensure that the assertions below are going to do something
        for record in all_checks:
            # no need to join with a '/' because the CFN runner adds it to the start of the file path
            self.assertEqual(record.repo_file_path, f'/{dir_rel_path}{record.file_path}')

    def test_record_relative_path_with_abs_dir(self):

        # test whether the record's repo_file_path is correct, relative to the CWD (with a / at the start).

        # this is just constructing the scan dir as normal
        current_dir = os.path.dirname(os.path.realpath(__file__))
        scan_dir_path = os.path.join(current_dir, "resources")

        dir_rel_path = os.path.relpath(scan_dir_path).replace('\\', '/')

        dir_abs_path = os.path.abspath(scan_dir_path)

        runner = Runner(db_connector=self.db_connector())
        checks_allowlist = ['CKV_AWS_20']
        report = runner.run(root_folder=dir_abs_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework='cloudformation', checks=checks_allowlist))

        all_checks = report.failed_checks + report.passed_checks
        self.assertGreater(len(all_checks), 0)  # ensure that the assertions below are going to do something
        for record in all_checks:
            # no need to join with a '/' because the CFN runner adds it to the start of the file path
            self.assertEqual(record.repo_file_path, f'/{dir_rel_path}{record.file_path}')

    def test_record_relative_path_with_relative_file(self):

        # test whether the record's repo_file_path is correct, relative to the CWD (with a / at the start).

        # this is just constructing the scan dir as normal
        current_dir = os.path.dirname(os.path.realpath(__file__))
        scan_file_path = os.path.join(current_dir, "resources", "success.json")

        # this is the relative path to the file to scan (what would actually get passed to the -f arg)
        file_rel_path = os.path.relpath(scan_file_path)

        runner = Runner(db_connector=self.db_connector())
        checks_allowlist = ['CKV_AWS_20']
        report = runner.run(root_folder=None, external_checks_dir=None, files=[file_rel_path],
                            runner_filter=RunnerFilter(framework='cloudformation', checks=checks_allowlist))

        all_checks = report.failed_checks + report.passed_checks
        self.assertGreater(len(all_checks), 0)  # ensure that the assertions below are going to do something
        for record in all_checks:
            # no need to join with a '/' because the CFN runner adds it to the start of the file path
            self.assertEqual(record.repo_file_path, f'/{file_rel_path}')

    def test_record_relative_path_with_abs_file(self):

        # test whether the record's repo_file_path is correct, relative to the CWD (with a / at the start).

        # this is just constructing the scan dir as normal
        current_dir = os.path.dirname(os.path.realpath(__file__))
        scan_file_path = os.path.join(current_dir, "resources", "success.json")

        file_rel_path = os.path.relpath(scan_file_path)
        file_abs_path = os.path.abspath(scan_file_path)

        runner = Runner(db_connector=self.db_connector())
        checks_allowlist = ['CKV_AWS_20']
        report = runner.run(root_folder=None, external_checks_dir=None, files=[file_abs_path],
                            runner_filter=RunnerFilter(framework='cloudformation', checks=checks_allowlist))

        all_checks = report.failed_checks + report.passed_checks
        self.assertGreater(len(all_checks), 0)  # ensure that the assertions below are going to do something
        for record in all_checks:
            # no need to join with a '/' because the CFN runner adds it to the start of the file path
            self.assertEqual(record.repo_file_path, f'/{file_rel_path}')

    def test_record_includes_custom_guideline(self):
        custom_guideline_url = "https://my.custom.url"
        custom_check_id = "MY_CUSTOM_CHECK"

        cfn_registry.checks = defaultdict(list)

        class AnyFailingCheck(BaseResourceCheck):
            def __init__(self, *_, **__) -> None:
                super().__init__(
                    "this should fail",
                    custom_check_id,
                    [CheckCategories.ENCRYPTION],
                    ["AWS::SQS::Queue"],
                    guideline=custom_guideline_url
                )

            def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
                return CheckResult.FAILED

        AnyFailingCheck()
        scan_file_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), "resources", "fail.yaml")

        report = Runner(db_connector=self.db_connector()).run(
            None,
            files=[scan_file_path],
            runner_filter=RunnerFilter(framework='cloudformation', checks=[custom_check_id])
        )

        self.assertEqual(report.failed_checks[0].guideline, custom_guideline_url)

    def test_get_tags(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        scan_file_path = os.path.join(current_dir, "resources", "tags.yaml")

        definitions, _ = parse(scan_file_path)

        resource_name = 'DataBucket'
        resource = definitions['Resources'][resource_name]
        entity = {resource_name: resource}
        entity_tags = cfn_utils.get_resource_tags(entity)

        self.assertEqual(len(entity_tags), 4)
        tags = {
            'Simple': 'Value',
            'Name': '${AWS::AccountId}-data',
            'Environment': 'long-form-sub-${account}',
            'Account': 'long-form-sub-${account}'
        }

        for name, value in tags.items():
            self.assertEqual(entity_tags[name], value)

        resource_name = 'NoTags'
        resource = definitions['Resources'][resource_name]
        entity = {resource_name: resource}
        entity_tags = cfn_utils.get_resource_tags(entity)

        self.assertIsNone(entity_tags)

        'TerraformServerAutoScalingGroup'
        resource_name = 'TerraformServerAutoScalingGroup'
        resource = definitions['Resources'][resource_name]
        entity = {resource_name: resource}
        entity_tags = cfn_utils.get_resource_tags(entity)
        if '__file__' in entity_tags:
            del entity_tags['__file__']

        self.assertDictEqual(
            entity_tags,
            {
                "Name": "TF-FulfillmentServer",
                "terraform-server-tag-key": "terraform-server-tag-value",
            }
        )

        resource_name = 'EKSClusterNodegroup'
        resource = definitions['Resources'][resource_name]
        entity = {resource_name: resource}
        entity_tags = cfn_utils.get_resource_tags(entity)
        if '__file__' in entity_tags:
            del entity_tags['__file__']

        self.assertDictEqual(
            entity_tags,
            {
                'Name': '{\'Ref\': \'ClusterName\'}-EKS-{\'Ref\': \'NodeGroupName\'}',
            }
        )

    def test_wrong_check_imports(self):
        wrong_imports = ["arm", "dockerfile", "helm", "kubernetes", "serverless", "terraform"]
        ignore_files = ["BaseCloudsplainingIAMCheck.py", "ECRPolicy.py"]
        check_imports = []

        checks_path = Path(inspect.getfile(Runner)).parent.joinpath("checks")
        for file in checks_path.rglob("*.py"):
            if file.name in ignore_files:
                continue

            with file.open() as f:
                instructions = dis.get_instructions(f.read())
                import_names = [instr.argval for instr in instructions if "IMPORT_NAME" == instr.opname]

                for import_name in import_names:
                    wrong_import = next((import_name for x in wrong_imports if x in import_name), None)
                    if wrong_import:
                        check_imports.append({file.name: wrong_import})

        assert len(check_imports) == 0, f"Wrong imports were added: {check_imports}"

    @pytest.mark.skip("No graph checks implemented yet for cloudformation")
    def test_run_graph_checks(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        scan_dir_path = os.path.join(current_dir, "../graph/checks/resources/MSKClusterLogging")


        dir_abs_path = os.path.abspath(scan_dir_path)

        report = Runner(db_connector=self.db_connector()).run(
            root_folder=dir_abs_path,
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["cloudformation"], download_external_modules=False)
        )

        self.assertGreater(len(report.failed_checks), 0)
        self.assertGreater(len(report.passed_checks), 0)

    def test_external_data(self):
        dir_abs_path = os.path.dirname(os.path.realpath(__file__))

        definitions = {
            f'{dir_abs_path}/s3.yaml': {
                'Resources': {
                    'MySourceQueue': {
                        'Type': 'AWS::SQS::Queue',
                        'Properties': {
                            'KmsMasterKeyId': 'kms_id',
                            '__startline__': 17,
                            '__endline__': 22,
                            'resource_type': 'AWS::SQS::Queue'
                        }
                    },
                    'MyDB': {
                        'Type': 'AWS::RDS::DBInstance',
                        'Properties': {
                            'DBName': 'db',
                            'DBInstanceClass': 'db.t3.micro',
                            'Engine': 'mysql',
                            'MasterUsername': 'master',
                            'MasterUserPassword': 'password',  # checkov:skip=CKV_SECRET_6 test secret
                            '__startline__': 23,
                            '__endline__': 32,
                            'resource_type': 'AWS::RDS::DBInstance'
                        }
                    }
                }
            }
        }

        context = {
            f"{dir_abs_path}/s3.yaml": {
                "Parameters": {
                    "KmsMasterKeyId": {
                        "start_line": 5,
                        "end_line": 9,
                        "code_lines": [
                            (5, '    "KmsMasterKeyId": {\n'),
                            (6, '      "Description": "Company Name",\n'),
                            (7, '      "Type": "String",\n'),
                            (8, '      "Default": "kms_id"\n'),
                            (9, "    },\n"),
                        ],
                    },
                    "DBName": {
                        "start_line": 10,
                        "end_line": 14,
                        "code_lines": [
                            (10, '    "DBName": {\n'),
                            (11, '      "Description": "Name of the Database",\n'),
                            (12, '      "Type": "String",\n'),
                            (13, '      "Default": "db"\n'),
                            (14, "    }\n"),
                        ],
                    },
                },
                "Resources": {
                    "MySourceQueue": {
                        "start_line": 17,
                        "end_line": 22,
                        "code_lines": [
                            (17, '    "MySourceQueue": {\n'),
                            (18, '      "Type": "AWS::SQS::Queue",\n'),
                            (19, '      "Properties": {\n'),
                            (20, '        "KmsMasterKeyId": { "Ref": "KmsMasterKeyId" }\n'),
                            (21, "      }\n"),
                            (22, "    },\n"),
                        ],
                        "skipped_checks": [],
                    },
                    "MyDB": {
                        "start_line": 23,
                        "end_line": 32,
                        "code_lines": [
                            (23, '    "MyDB": {\n'),
                            (24, '      "Type": "AWS::RDS::DBInstance",\n'),
                            (25, '      "Properties": {\n'),
                            (26, '        "DBName": { "Ref": "DBName" },\n'),
                            (27, '        "DBInstanceClass": "db.t3.micro",\n'),
                            (28, '        "Engine": "mysql",\n'),
                            (29, '        "MasterUsername": "master",\n'),
                            (30, '        "MasterUserPassword": "password"\n'),
                            (31, "      }\n"),
                            (32, "    }\n"),
                        ],
                        "skipped_checks": [],
                    },
                },
                "Outputs": {
                    "DBAppPublicDNS": {
                        "start_line": 35,
                        "end_line": 38,
                        "code_lines": [
                            (35, '    "DBAppPublicDNS": {\n'),
                            (36, '      "Description": "DB App Public DNS Name",\n'),
                            (37, '      "Value": { "Fn::GetAtt" : [ "MyDB", "PublicDnsName" ] }\n'),
                            (38, "    }\n"),
                        ],
                    }
                },
            }
        }
        breadcrumbs = {}
        runner = Runner(db_connector=self.db_connector())
        runner.set_external_data(definitions, context, breadcrumbs)
        report = Report('cloudformation')
        runner.check_definitions(root_folder=dir_abs_path, runner_filter=RunnerFilter(framework='cloudformation', download_external_modules=False), report=report)
        self.assertEqual(len(report.passed_checks), 2)
        self.assertEqual(len(report.failed_checks), 4)
        pass

    def test_breadcrumbs_report(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        scan_dir_path = os.path.join(current_dir, "../graph/graph_builder/resources/variable_rendering/render_params")

        dir_abs_path = os.path.abspath(scan_dir_path)

        runner = Runner(db_connector=self.db_connector())
        report = runner.run(root_folder=dir_abs_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework='cloudformation', download_external_modules=False, checks=["CKV_AWS_21"]))

        self.assertEqual(1, len(report.failed_checks))
        self.assertIsNotNone(report.failed_checks[0].breadcrumbs)
        self.assertIsNotNone(report.failed_checks[0].breadcrumbs.get("VersioningConfiguration.Status"))

    def test_parsing_invalid_properties_yaml(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        scan_file_path = os.path.join(current_dir, "resources", "invalid_properties.yaml")
        runner = Runner(db_connector=self.db_connector())
        report = runner.run(root_folder=None, external_checks_dir=None, files=[scan_file_path],
                            runner_filter=RunnerFilter(framework='cloudformation'))
        self.assertEqual(report.parsing_errors, [scan_file_path])

    def test_parsing_invalid_properties_json(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        scan_file_path = os.path.join(current_dir, "resources", "invalid_properties.json")
        runner = Runner(db_connector=self.db_connector())
        report = runner.run(root_folder=None, external_checks_dir=None, files=[scan_file_path],
                            runner_filter=RunnerFilter(framework='cloudformation'))
        self.assertEqual(report.parsing_errors, [scan_file_path])

    def test_parsing_no_properties_yaml(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        scan_file_path = os.path.join(current_dir, "resources", "no_properties.yaml")
        runner = Runner(db_connector=self.db_connector())
        report = runner.run(root_folder=None, external_checks_dir=None, files=[scan_file_path],
                            runner_filter=RunnerFilter(framework='cloudformation'))
        self.assertEqual(len(report.failed_checks), 3)
        self.assertEqual(len(report.passed_checks), 3)

    def test_runner_honors_enforcement_rules(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        scan_file_path = os.path.join(current_dir, "resources", "graph.yaml")
        runner = Runner(db_connector=self.db_connector())
        filter = RunnerFilter(framework=['cloudformation'], use_enforcement_rules=True)
        # this is not quite a true test, because the checks don't have severities. However, this shows that the check registry
        # passes the report type properly to RunnerFilter.should_run_check, and we have tests for that method
        filter.enforcement_rule_configs = {CheckType.CLOUDFORMATION: Severities[BcSeverities.OFF]}
        report = runner.run(root_folder=None, external_checks_dir=None, files=[scan_file_path], runner_filter=filter)
        self.assertEqual(len(report.failed_checks), 0)
        self.assertEqual(len(report.passed_checks), 0)
        self.assertEqual(len(report.skipped_checks), 0)
        self.assertEqual(len(report.parsing_errors), 0)

    def test_parsing_no_properties_json(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        scan_file_path = os.path.join(current_dir, "resources", "no_properties.json")
        runner = Runner(db_connector=self.db_connector())
        report = runner.run(root_folder=None, external_checks_dir=None, files=[scan_file_path],
                            runner_filter=RunnerFilter(framework='cloudformation'))
        self.assertEqual(len(report.failed_checks), 3)
        self.assertEqual(len(report.passed_checks), 3)

    def test_parsing_error_yaml(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        scan_file_path = os.path.join(current_dir, "resources", "invalid.yaml")
        runner = Runner(db_connector=self.db_connector())
        report = runner.run(root_folder=None, external_checks_dir=None, files=[scan_file_path],
                            runner_filter=RunnerFilter(framework='cloudformation'))
        self.assertEqual(report.parsing_errors, [scan_file_path])

    def test_double_statement(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        scan_file_path = os.path.join(current_dir, "resources", "double_statement_cloudsplaining.yml")
        runner = Runner(db_connector=self.db_connector())
        report = runner.run(root_folder=None, external_checks_dir=None, files=[scan_file_path],
                            runner_filter=RunnerFilter(framework=['cloudformation']))
        self.assertEqual(len(report.failed_checks), 5)
        id_and_key = [(x.check_id, x.check_result.get('evaluated_keys')) for x in report.failed_checks]
        self.assertIn(('CKV_AWS_107', ['Properties/PolicyDocument/Statement/[1]/Action']), id_and_key)
        self.assertIn(('CKV_AWS_111', ['Properties/PolicyDocument/Statement/[0]/Action/[1]/']), id_and_key)

    def test_skip_sub_dict_json(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        scan_file_path = os.path.join(current_dir, "resources", "skip_sub_dict.json")
        runner = Runner(db_connector=self.db_connector())
        report = runner.run(root_folder=None, external_checks_dir=None, files=[scan_file_path],
                            runner_filter=RunnerFilter(framework='cloudformation'))
        self.assertEqual(len(report.failed_checks), 0)

    def test_parsing_error_json(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        scan_file_path = os.path.join(current_dir, "resources", "invalid.json")
        runner = Runner(db_connector=self.db_connector())
        report = runner.run(root_folder=None, external_checks_dir=None, files=[scan_file_path],
                            runner_filter=RunnerFilter(framework='cloudformation'))
        self.assertEqual(report.parsing_errors, [scan_file_path])

    def test_parse_relevant_files_only(self):
        definitions, _ = create_definitions(None, ['main.tf'])
        # just check that we skip the file and return normally
        self.assertNotIn('main.tf', definitions)

    def test_record_includes_severity(self):
        custom_check_id = "MY_CUSTOM_CHECK"

        cfn_registry.checks = defaultdict(list)

        class AnyFailingCheck(BaseResourceCheck):
            def __init__(self, *_, **__) -> None:
                super().__init__(
                    "this should fail",
                    custom_check_id,
                    [CheckCategories.ENCRYPTION],
                    ["AWS::SQS::Queue"]
                )

            def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
                return CheckResult.FAILED

        check = AnyFailingCheck()
        check.severity = Severities[BcSeverities.LOW]
        scan_file_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), "resources", "fail.yaml")

        report = Runner(db_connector=self.db_connector()).run(
            None,
            files=[scan_file_path],
            runner_filter=RunnerFilter(framework=['cloudformation'], checks=[custom_check_id])
        )

        self.assertEqual(report.failed_checks[0].severity, Severities[BcSeverities.LOW])

    def test_severity_check_filter_omit(self):
        custom_check_id = "MY_CUSTOM_CHECK"

        cfn_registry.checks = defaultdict(list)

        class AnyFailingCheck(BaseResourceCheck):
            def __init__(self, *_, **__) -> None:
                super().__init__(
                    "this should fail",
                    custom_check_id,
                    [CheckCategories.ENCRYPTION],
                    ["AWS::SQS::Queue"]
                )

            def scan_resource_conf(self, conf: Dict[str, Any], entity_type: str) -> CheckResult:
                return CheckResult.FAILED

        check = AnyFailingCheck()
        check.severity = Severities[BcSeverities.LOW]

        runner = Runner(db_connector=self.db_connector())
        checks_allowlist = ['MEDIUM']
        scan_file_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), "resources", "fail.yaml")
        report = runner.run(None, files=[scan_file_path], external_checks_dir=None,
                            runner_filter=RunnerFilter(framework='cloudformation', checks=checks_allowlist))

        all_checks = report.failed_checks + report.passed_checks
        self.assertFalse(any(c.check_id == custom_check_id for c in all_checks))

    def test_severity_check_filter_include(self):

        custom_check_id = "MY_CUSTOM_CHECK"

        cfn_registry.checks = defaultdict(list)

        class AnyFailingCheck(BaseResourceCheck):
            def __init__(self, *_, **__) -> None:
                super().__init__(
                    "this should fail",
                    custom_check_id,
                    [CheckCategories.ENCRYPTION],
                    ["AWS::SQS::Queue"]
                )

            def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
                return CheckResult.FAILED

        check = AnyFailingCheck()
        check.severity = Severities[BcSeverities.HIGH]

        runner = Runner(db_connector=self.db_connector())
        checks_allowlist = ['MEDIUM']
        scan_file_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), "resources", "fail.yaml")
        report = runner.run(None, files=[scan_file_path], external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['cloudformation'], checks=checks_allowlist))

        all_checks = report.failed_checks + report.passed_checks
        self.assertTrue(any(c.check_id == custom_check_id for c in all_checks))

    def test_severity_skip_check_filter_omit(self):

        custom_check_id = "MY_CUSTOM_CHECK"

        cfn_registry.checks = defaultdict(list)

        class AnyFailingCheck(BaseResourceCheck):
            def __init__(self, *_, **__) -> None:
                super().__init__(
                    "this should fail",
                    custom_check_id,
                    [CheckCategories.ENCRYPTION],
                    ["AWS::SQS::Queue"]
                )

            def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
                return CheckResult.FAILED

        check = AnyFailingCheck()
        check.severity = Severities[BcSeverities.LOW]

        runner = Runner(db_connector=self.db_connector())
        checks_denylist = ['MEDIUM']
        scan_file_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), "resources", "fail.yaml")
        report = runner.run(None, files=[scan_file_path], external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['cloudformation'], skip_checks=checks_denylist))

        all_checks = report.failed_checks + report.passed_checks
        self.assertFalse(any(c.check_id == custom_check_id for c in all_checks))

    def test_severity_skip_check_filter_include(self):

        custom_check_id = "MY_CUSTOM_CHECK"

        cfn_registry.checks = defaultdict(list)

        class AnyFailingCheck(BaseResourceCheck):
            def __init__(self, *_, **__) -> None:
                super().__init__(
                    "this should fail",
                    custom_check_id,
                    [CheckCategories.ENCRYPTION],
                    ["AWS::SQS::Queue"]
                )

            def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
                return CheckResult.FAILED

        check = AnyFailingCheck()
        check.severity = Severities[BcSeverities.HIGH]

        runner = Runner(db_connector=self.db_connector())
        checks_denylist = ['MEDIUM']
        scan_file_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), "resources", "fail.yaml")
        report = runner.run(None, files=[scan_file_path], external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['cloudformation'], skip_checks=checks_denylist))

        all_checks = report.failed_checks + report.passed_checks
        self.assertTrue(any(c.check_id == custom_check_id for c in all_checks))

    def test_graph_check_suppressions(self):
        # given
        test_file = RESOURCES_DIR / "suppress_graph_check.yaml"

        # when
        report = Runner(db_connector=self.db_connector()).run(
            root_folder=None,
            files=[str(test_file)],
            runner_filter=RunnerFilter(framework=["cloudformation"], checks=["CKV2_AWS_33"]),
        )

        # then
        self.assertEqual(len(report.failed_checks), 0)
        self.assertEqual(len(report.passed_checks), 0)
        self.assertEqual(len(report.skipped_checks), 2)

    def tearDown(self):
        cfn_registry.checks = self.orig_checks


if __name__ == '__main__':
    unittest.main()



from __future__ import annotations

from pathlib import Path

import pytest
from time_machine import travel

from checkov.github.runner import Runner
from checkov.runner_filter import RunnerFilter
from tests.common.graph.checks.test_yaml_policies_base import load_yaml_data

BASE_DIR = Path(__file__).parent
CHECK_ID_MAP: "dict[str, str]" = {}  # will be filled via setup()


def test_GithubBranchDisallowDeletions():
    run_check(base_path=BASE_DIR / "branch_security", check="GithubBranchDisallowDeletions")


def test_GithubBranchDismissStaleReviews():
    run_check(base_path=BASE_DIR / "branch_security", check="GithubBranchDismissStaleReviews")


def test_GithubBranchDismissalRestrictions():
    run_check(base_path=BASE_DIR / "branch_security", check="GithubBranchDismissalRestrictions")


def test_GithubBranchRequireCodeOwnerReviews():
    run_check(base_path=BASE_DIR / "branch_security", check="GithubBranchRequireCodeOwnerReviews")


def test_GithubBranchRequireConversationResolution():
    run_check(base_path=BASE_DIR / "branch_security", check="GithubBranchRequireConversationResolution")


def test_GithubBranchRequirePushRestrictions():
    run_check(base_path=BASE_DIR / "branch_security", check="GithubBranchRequirePushRestrictions")


def test_GithubBranchRequireStatusChecks():
    run_check(base_path=BASE_DIR / "branch_security", check="GithubBranchRequireStatusChecks")


def test_GithubRequire2Approvals():
    run_check(base_path=BASE_DIR / "branch_security", check="GithubRequire2Approvals")


@travel("2022-12-05")
def test_GithubDisallowInactiveBranch60Days():
    run_check(base_path=BASE_DIR / "branch_security", check="GithubDisallowInactiveBranch60Days")


def test_GithubRequireUpdatedBranch():
    run_check(base_path=BASE_DIR / "branch_security", check="GithubRequireUpdatedBranch")


def test_GithubPublicRepositoryCreationIsLimited():
    run_check(base_path=BASE_DIR / "repo_management", check="GithubPublicRepositoryCreationIsLimited")


def test_GithubInternalRepositoryCreationIsLimited():
    run_check(base_path=BASE_DIR / "repo_management", check="GithubInternalRepositoryCreationIsLimited")


def test_GithubPrivateRepositoryCreationIsLimited():
    run_check(base_path=BASE_DIR / "repo_management", check="GithubPrivateRepositoryCreationIsLimited")


def test_GithubMinimumAdminsInOrganization():
    run_check(base_path=BASE_DIR / "contribution_access", check="GithubMinimumAdminsInOrganization")


def test_GithubRequireStrictBasePermissionsRepository():
    run_check(base_path=BASE_DIR / "contribution_access", check="GithubRequireStrictBasePermissionsRepository")


def test_GithubRequireOrganizationIsVerified():
    run_check(base_path=BASE_DIR / "contribution_access", check="GithubRequireOrganizationIsVerified")


@pytest.fixture(autouse=True)
def setup():
    global CHECK_ID_MAP
    runner = Runner()
    registry = runner.import_registry()
    CHECK_ID_MAP = {check.__class__.__name__: check.id for entity, check in registry.all_checks()}


def run_check(base_path: Path, check: str) -> None:
    # set path where to find test files
    test_dir_path = base_path / check

    # setup GitHub configuration runner
    runner = Runner()
    runner.github.github_conf_dir_path = str(test_dir_path)

    # run actual check
    report = runner.run(runner_filter=RunnerFilter(checks=CHECK_ID_MAP[check]))

    # get actual results
    summary = report.get_summary()
    passed_checks = {check.file_path.lstrip("/") for check in report.passed_checks}
    failed_checks = {check.file_path.lstrip("/") for check in report.failed_checks}

    # get expected results
    expected = load_yaml_data(dir_path=test_dir_path, source_file_name="expected.yaml")

    # make sure it is a dict
    assert isinstance(expected, dict)

    expected_to_pass = expected.get("pass") or []
    expected_to_fail = expected.get("fail") or []

    # check, if results are correct
    assert summary["passed"] == len(expected_to_pass)
    assert summary["failed"] == len(expected_to_fail)
    assert summary["skipped"] == 0
    assert summary["parsing_errors"] == 0

    assert passed_checks == set(expected_to_pass)
    assert failed_checks == set(expected_to_fail)



import base64
import os
import random
import unittest
import uuid
from unittest import mock
from checkov.common.bridgecrew.bc_source import get_source_type
from checkov.common.bridgecrew.integration_features.features.policy_metadata_integration import \
    PolicyMetadataIntegration
from checkov.common.bridgecrew.platform_integration import BcPlatformIntegration
from checkov.common.bridgecrew.severities import Severities, BcSeverities
from checkov.common.checks.base_check_registry import BaseCheckRegistry


class TestBCApiUrl(unittest.TestCase):

    @mock.patch.dict(os.environ, {'BC_API_URL': 'https://www.bridgecrew.cloud'})
    def test_overriding_bc_api_url(self):
        instance = BcPlatformIntegration()
        self.assertEqual(instance.api_url, "https://www.bridgecrew.cloud")

    @staticmethod
    def get_random_string():
        len = random.randrange(5, 50)
        chars = []
        for i in range(0, len):
            chars.append(chr(random.randrange(32, 127)))
        return ''.join(chars)

    def test_is_token_valid(self):
        uuids = []
        for i in range(0, 1000):
            uuids.append(str(uuid.uuid4()))

        # validate BC API keys
        for u in uuids:
            self.assertTrue(BcPlatformIntegration.is_token_valid(u))

        # generate Prisma access keys, which are UUIDs (just reuse the ones from above),
        # and secret keys, which are b64 encoded strings
        for i in range(0, len(uuids)):
            string_to_encode = self.get_random_string()
            encoded = base64.b64encode(bytes(string_to_encode, 'utf-8'))
            uuids[i] = uuids[i] + '::' + encoded.decode('utf-8')

        for u in uuids:
            self.assertTrue(BcPlatformIntegration.is_token_valid(u))

        uuid_str = str(uuid.uuid4())
        b64_str = base64.b64encode(bytes(self.get_random_string(), 'utf-8')).decode('utf-8')
        self.assertFalse(BcPlatformIntegration.is_token_valid(f'{uuid_str}{b64_str}'))
        self.assertFalse(BcPlatformIntegration.is_token_valid(f'{uuid_str}:{b64_str}'))
        self.assertFalse(BcPlatformIntegration.is_token_valid(f'{uuid_str}:::{b64_str}'))
        self.assertFalse(BcPlatformIntegration.is_token_valid(f'{uuid_str}::'))
        self.assertFalse(BcPlatformIntegration.is_token_valid(f'::{b64_str}'))
        self.assertFalse(BcPlatformIntegration.is_token_valid(''))
        self.assertFalse(BcPlatformIntegration.is_token_valid('1234::56789'))
        self.assertFalse(BcPlatformIntegration.is_token_valid(f'{uuid_str}::56789'))
        self.assertFalse(BcPlatformIntegration.is_token_valid(f'1234::{b64_str}'))

    def test_overriding_pc_api_url(self):
        instance = BcPlatformIntegration()
        instance.setup_bridgecrew_credentials(
            repo_id="bridgecrewio/checkov",
            prisma_api_url="https://api0.prismacloud.io",
            source=get_source_type('disabled')
        )
        self.assertEqual(instance.api_url, "https://api0.prismacloud.io/bridgecrew")
        self.assertEqual(instance.prisma_api_url, "https://api0.prismacloud.io")

    def test_no_overriding_api_url(self):
        instance = BcPlatformIntegration()
        self.assertEqual(instance.api_url, "https://api0.prismacloud.io/bridgecrew")

    def test_skip_mapping_default(self):
        # Default is False so mapping is obtained
        instance = BcPlatformIntegration()
        instance.api_url = 'https://www.bridgecrew.cloud/v1'
        instance.setup_http_manager()
        instance.get_public_run_config()
        metadata_integration = PolicyMetadataIntegration(instance)
        metadata_integration.bc_integration = instance
        metadata_integration.pre_scan()
        self.assertIsNotNone(metadata_integration.check_metadata)
        self.assertGreater(len(metadata_integration.check_metadata), 0)

    def test_skip_mapping_true(self):
        instance = BcPlatformIntegration()
        instance.skip_download = True
        instance.setup_http_manager()
        instance.get_public_run_config()
        metadata_integration = PolicyMetadataIntegration(instance)
        metadata_integration.bc_integration = instance
        metadata_integration.pre_scan()
        self.assertIsNotNone(metadata_integration.check_metadata)
        self.assertDictEqual({}, metadata_integration.check_metadata)

    def test_metadata_bc_key(self):
        instance = BcPlatformIntegration()
        instance.bc_api_key = '00000000-0000-0000-0000-000000000000'
        instance.customer_run_config_response = mock_customer_run_config()
        metadata_integration = PolicyMetadataIntegration(instance)
        metadata_integration.bc_integration = instance
        metadata_integration.pre_scan()
        all_checks = BaseCheckRegistry.get_all_registered_checks()
        check_same_severity = next((check for check in all_checks if check.id == "CKV_AWS_15"), None)
        check_different_severity = next((check for check in all_checks if check.id == "CKV_AWS_40"), None)
        check_no_desc_title = next((check for check in all_checks if check.id == "CKV_AWS_53"), None)

        self.assertEqual(check_same_severity.name, 'Ensure IAM password policy requires at least one uppercase letter')
        self.assertEqual(check_same_severity.severity, Severities[BcSeverities.INFO])
        self.assertEqual(check_different_severity.severity, Severities[BcSeverities.CRITICAL])
        self.assertEqual(check_no_desc_title.severity, Severities[BcSeverities.MEDIUM])

    def test_metadata_prisma_key(self):
        instance = BcPlatformIntegration()
        instance.bc_api_key = '00000000-0000-0000-0000-000000000000::1234=='
        instance.customer_run_config_response = mock_customer_run_config()
        metadata_integration = PolicyMetadataIntegration(instance)
        metadata_integration.bc_integration = instance
        metadata_integration.pre_scan()
        all_checks = BaseCheckRegistry.get_all_registered_checks()
        check_same_severity = next((check for check in all_checks if check.id == "CKV_AWS_15"), None)
        check_different_severity = next((check for check in all_checks if check.id == "CKV_AWS_40"), None)
        check_no_desc_title = next((check for check in all_checks if check.id == "CKV_AWS_53"), None)

        self.assertEqual(check_same_severity.name, 'AWS IAM password policy does not have an uppercase character')
        self.assertEqual(check_different_severity.name, 'AWS IAM policy attached to users')
        self.assertEqual(check_no_desc_title.name, 'Ensure S3 bucket has block public ACLS enabled')
        self.assertEqual(check_same_severity.severity, Severities[BcSeverities.INFO])
        self.assertEqual(check_different_severity.severity, Severities[BcSeverities.HIGH])
        self.assertEqual(check_no_desc_title.severity, None)

    def test_should_upload(self):
        self.assertFalse(get_source_type('vscode').upload_results)
        self.assertTrue(get_source_type('cli').upload_results)
        self.assertTrue(get_source_type('xyz').upload_results)
        self.assertTrue(get_source_type(None).upload_results)

    def test_run_config_url(self):
        instance = BcPlatformIntegration()
        instance.repo_id = 'owner/repo'
        instance.bc_api_key = '00000000-0000-0000-0000-000000000000'
        self.assertTrue(instance.get_run_config_url().endswith('/runConfiguration?module=bc&enforcementv2=true&repoId=owner/repo'))
        instance.bc_api_key = '00000000-0000-0000-0000-000000000000::1234=='
        self.assertTrue(instance.get_run_config_url().endswith('/runConfiguration?module=pc&enforcementv2=true&repoId=owner/repo'))
        instance.repo_id = 'encode/më'
        self.assertTrue(instance.get_run_config_url().endswith('/runConfiguration?module=pc&enforcementv2=true&repoId=encode/m%C3%AB'))

    def test_is_valid_policy_filter(self):
        instance = BcPlatformIntegration()
        instance.bc_api_key = '00000000-0000-0000-0000-000000000000::1234=='
        instance.customer_run_config_response = mock_customer_run_config()
        self.assertTrue(instance.is_valid_policy_filter(policy_filter=[('policy.label', 'CODE')],
                                                        valid_filters=mock_prisma_policy_filter_response()))
        self.assertFalse(instance.is_valid_policy_filter(policy_filter=[('policy.labels', 'CODE')],
                                                        valid_filters=mock_prisma_policy_filter_response()))
        self.assertFalse(instance.is_valid_policy_filter(policy_filter=[('policy.label', 'CODE'), ('not', 'allowed')],
                                                        valid_filters=mock_prisma_policy_filter_response()))
        self.assertFalse(instance.is_valid_policy_filter(policy_filter=[],
                                                         valid_filters=mock_prisma_policy_filter_response()))
        self.assertFalse(instance.is_valid_policy_filter(policy_filter=[('policy.label', 'A'), ('policy.label', 'B')], valid_filters={}))

    def test_add_static_policy_filters(self):
        self.assertListEqual(BcPlatformIntegration.add_static_policy_filters([]), [('policy.enabled', 'true'), ('policy.subtype', 'build')])
        self.assertListEqual(BcPlatformIntegration.add_static_policy_filters([('policy.enabled', 'true')]), [('policy.enabled', 'true'), ('policy.subtype', 'build')])
        self.assertListEqual(BcPlatformIntegration.add_static_policy_filters([('policy.enabled', 'true'), ('policy.subtype', 'build')]), [('policy.enabled', 'true'), ('policy.subtype', 'build')])
        self.assertListEqual(BcPlatformIntegration.add_static_policy_filters([('policy.label', 'xyz')]), [('policy.label', 'xyz'), ('policy.enabled', 'true'), ('policy.subtype', 'build')])
        self.assertListEqual(BcPlatformIntegration.add_static_policy_filters([('policy.label', 'xyz'), ('policy.enabled', 'true')]), [('policy.label', 'xyz'), ('policy.enabled', 'true'), ('policy.subtype', 'build')])
        self.assertListEqual(BcPlatformIntegration.add_static_policy_filters([('policy.enabled', 'true'), ('policy.label', 'xyz'), ('policy.subtype', 'build')]), [('policy.enabled', 'true'), ('policy.label', 'xyz'), ('policy.subtype', 'build')])

    def test_setup_on_prem(self):
        instance = BcPlatformIntegration()

        instance.customer_run_config_response = None
        instance.setup_on_prem()
        self.assertFalse(instance.on_prem)

        instance.customer_run_config_response = {}
        instance.setup_on_prem()
        self.assertFalse(instance.on_prem)

        instance.customer_run_config_response = {
            'tenantConfig': {}
        }
        instance.setup_on_prem()
        self.assertFalse(instance.on_prem)

        instance.customer_run_config_response = {
            'tenantConfig': {
                'preventCodeUploads': False
            }
        }
        instance.setup_on_prem()
        self.assertFalse(instance.on_prem)

        instance.customer_run_config_response = {
            'tenantConfig': {
                'preventCodeUploads': True
            }
        }
        instance.setup_on_prem()
        self.assertTrue(instance.on_prem)


def mock_customer_run_config():
    return {
        "policyMetadata": {
            "CKV_AWS_15": {
                "id": "BC_AWS_IAM_5",
                "title": "Ensure IAM password policy requires at least one uppercase letter",
                "guideline": "https://docs.bridgecrew.io/docs/iam_5",
                "severity": "INFO",
                "pcSeverity": "INFO",
                "category": "IAM",
                "checkovId": "CKV_AWS_15",
                "constructiveTitle": "Ensure AWS IAM password policy has an uppercase character",
                "descriptiveTitle": "AWS IAM password policy does not have an uppercase character",
                "pcPolicyId": "31626ca9-f659-4d25-9d88-fa32262bbba7",
                "additionalPcPolicyIds": [
                    "31626ca9-f659-4d25-9d88-fa32262bbba7"
                ],
                "benchmarks": {}
            },
            "CKV_AWS_40": {
                "id": "BC_AWS_IAM_16",
                "title": "Ensure IAM policies are attached only to groups or roles",
                "guideline": "https://docs.bridgecrew.io/docs/iam_16-iam-policy-privileges-1",
                "severity": "CRITICAL",
                "pcSeverity": "HIGH",
                "category": "IAM",
                "checkovId": "CKV_AWS_40",
                "constructiveTitle": "Ensure IAM policies are only attached to Groups and Roles",
                "descriptiveTitle": "AWS IAM policy attached to users",
                "pcPolicyId": "2b7e07ba-56c8-42db-8db4-a4b65f5066c4",
                "additionalPcPolicyIds": [
                    "2b7e07ba-56c8-42db-8db4-a4b65f5066c4"
                ],
                "benchmarks": {}
            },
            "CKV_AWS_53": {
                "id": "BC_AWS_S3_19",
                "title": "Ensure S3 bucket has block public ACLS enabled",
                "guideline": "https://docs.bridgecrew.io/docs/bc_aws_s3_19",
                "severity": "MEDIUM",
                "pcSeverity": None,
                "category": "Storage",
                "checkovId": "CKV_AWS_53",
                "constructiveTitle": "Ensure S3 bucket has block public ACLS enabled",
                "descriptiveTitle": None,
                "pcPolicyId": "34064d53-1fd1-42e6-b075-45dce495caca",
                "additionalPcPolicyIds": [
                    "34064d53-1fd1-42e6-b075-45dce495caca"
                ],
                "benchmarks": {}
            }
        }
    }


def mock_prisma_policy_filter_response():
    return {
        "policy.name": {
            "options": [
                "CODE1: Ensure subnet is associated with NSG"
            ],
            "staticFilter": False
        },
        "policy.type": {
            "options": [
                "anomaly",
                "audit_event",
                "config",
                "data",
                "network"
            ],
            "staticFilter": True
        },
        "policy.label": {
            "options": [
                "CODE",
                "KARTIK",
                "CRITICAL"
            ],
            "staticFilter": False
        },
        "policy.complianceStandard": {
            "options": [],
            "staticFilter": False
        },
        "policy.complianceRequirement": {
            "options": [],
            "staticFilter": False
        },
        "policy.complianceSection": {
            "options": [],
            "staticFilter": False
        },
        "policy.category": {
            "options": [
                "incident",
                "risk"
            ],
            "staticFilter": True
        },
        "policy.class": {
            "options": [
                "behavioral",
                "exposure",
                "network_protection",
                "privileged_activity_monitoring",
                "vulnerabilities"
            ],
            "staticFilter": True
        },
        "policy.policyMode": {
            "options": [
                "redlock_default",
                "custom"
            ],
            "staticFilter": True
        },
        "policy.subtype": {
            "options": [
                "audit",
                "build",
                "data_classification",
                "dns",
                "malware",
                "network",
                "network_config",
                "network_event",
                "run",
                "run_and_build",
                "ueba"
            ],
            "staticFilter": True
        },
        "policy.enabled": {
            "options": [
                "true",
                "false"
            ],
            "staticFilter": True
        },
        "cloud.type": {
            "options": [
                "alibaba_cloud",
                "aws",
                "azure",
                "gcp",
                "oci"
            ],
            "staticFilter": True
        },
        "policy.severity": {
            "options": [
                "high",
                "medium",
                "low"
            ],
            "staticFilter": True
        },
        "policy.remediable": {
            "options": [
                "true",
                "false"
            ],
            "staticFilter": True
        }
    }


def mock_prisma_policies_response():
    return [
        {
            "policyId": "6960be11-e3a6-46cc-bf66-933c57c2af5d",
            "name": "AWS EBS volume region with encryption is disabled",
            "policyType": "config",
            "policySubTypes": [
                "run",
                "build"
            ],
            "systemDefault": True,
            "policyUpi": "PC-AWS-EC2-778",
            "description": "This policy identifies AWS regions in which new EBS volumes are getting created without any encryption. Encrypting data at rest reduces unintentional exposure of data stored in EBS volumes. It is recommended to configure EBS volume at the regional level so that every new EBS volume created in that region will be enabled with encryption by using a provided encryption key.",
            "severity": "medium",
            "rule": {
                "name": "AWS EBS volume region with encryption is disabled",
                "criteria": "7a951e9f-02d2-4d9f-9441-29b545084585",
                "parameters": {
                    "withIac": "true",
                    "savedSearch": "true"
                },
                "type": "Config",
                "children": [
                    {
                        "criteria": "{\"category\":\"Kubernetes\",\"resourceTypes\":[\"aws_ebs_encryption_by_default\"]}",
                        "type": "build",
                        "metadata": {
                            "checkovId": "CKV_AWS_106"
                        },
                        "recommendation": "Refer the documentation for more details,\nhttps://docs.bridgecrew.io/docs/ensure-kubernetes-secrets-are-encrypted-using-customer-master-keys-cmks-managed-in-aws-kms"
                    }
                ]
            },
            "recommendation": "To enable encryption at region level by default, follow below URL:\nhttps://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSEncryption.html#encryption-by-default\n\nAdditional Information:\n\nTo detect existing EBS volumes that are not encrypted ; refer Saved Search:\nAWS EBS volumes are not encrypted_RL\n\nTo detect existing EBS volumes that are not encrypted with CMK, refer Saved Search:\nAWS EBS volume not encrypted using Customer Managed Key_RL",
            "cloudType": "aws",
            "complianceMetadata": [
                {
                    "standardId": "a0ea1077-424f-45fd-994e-4caef6d4d9de",
                    "standardName": "AWS Foundational Security Best Practices standard",
                    "standardDescription": "AWS Foundational Security Best Practices standard",
                    "requirementId": "Protect",
                    "requirementName": "Protect",
                    "sectionId": "Data protection",
                    "sectionDescription": "Data protection",
                    "policyId": "6960be11-e3a6-46cc-bf66-933c57c2af5d",
                    "complianceId": "2ab6eea3-660b-48c4-a836-53347e702faf",
                    "sectionLabel": "Data protection",
                    "sectionViewOrder": 5,
                    "requirementViewOrder": 2,
                    "systemDefault": True,
                    "customAssigned": False
                }
            ],
            "labels": [
                "CODE",
                "KARTIK"
            ],
            "enabled": True,
            "createdOn": 1634832351154,
            "createdBy": "Prisma Cloud System Admin",
            "lastModifiedOn": 1654106204620,
            "lastModifiedBy": "User",
            "ruleLastModifiedOn": 1639652340813,
            "deleted": False,
            "owner": "Prisma Cloud",
            "policyMode": "redlock_default",
            "policyCategory": "risk",
            "policyClass": "exposure",
            "remediable": False
        },
        {
            "policyId": "c11ce08c-b93e-4e11-8d1c-e5a1339139d1",
            "name": "CUSTOM 3: Ensure subnet is associated with NSG",
            "policyType": "config",
            "policySubTypes": [
                "build"
            ],
            "systemDefault": False,
            "description": "Every subnet should be associated with NSG for controlling access to \nresources within the subnet.\n",
            "severity": "high",
            "rule": {
                "name": "CUSTOM 3: Ensure subnet is associated with NSG",
                "parameters": {
                    "withIac": "true",
                    "savedSearch": "false"
                },
                "type": "Config",
                "children": [
                    {
                        "type": "build",
                        "metadata": {
                            "code": "metadata:\n  name: 'CUSTOM 3: Ensure subnet is associated with NSG'\n  guidelines: \"Every subnet should be associated with NSG for controlling access to\\\n    \\ \\nresources within the subnet.\\n\"\n  category: networking\n  severity: high\nscope:\n  provider: azure\ndefinition:\n  and:\n  - cond_type: connection\n    resource_types:\n    - azurerm_subnet_network_security_group_association\n    connected_resource_types:\n    - azurerm_subnet\n    - azurerm_network_security_group\n    operator: exists\n  - cond_type: filter\n    attribute: resource_type\n    value:\n    - azurerm_subnet\n    operator: within\n"
                        },
                        "recommendation": ""
                    }
                ]
            },
            "recommendation": "",
            "cloudType": "azure",
            "labels": [
                "CODE",
                "KARTIK"
            ],
            "enabled": True,
            "createdOn": 1653999690681,
            "createdBy": "User",
            "lastModifiedOn": 1654065712108,
            "lastModifiedBy": "User",
            "ruleLastModifiedOn": 1653999690681,
            "deleted": False,
            "owner": "Tenant Name",
            "policyMode": "custom",
            "policyCategory": "risk",
            "policyClass": "exposure",
            "remediable": False
        },
        {
            "policyId": "0e4c576e-c934-4af3-8592-a53920e71ffb",
            "name": "CUSTOM POLICY2: Ensure subnet is associated with NSG",
            "policyType": "config",
            "policySubTypes": [
                "build"
            ],
            "systemDefault": False,
            "description": "Every subnet should be associated with NSG for controlling access to \nresources within the subnet.\n",
            "severity": "high",
            "rule": {
                "name": "CUSTOM POLICY2: Ensure subnet is associated with NSG",
                "parameters": {
                    "withIac": "true",
                    "savedSearch": "false"
                },
                "type": "Config",
                "children": [
                    {
                        "type": "build",
                        "metadata": {
                            "code": "metadata:\n  name: 'CUSTOM POLICY2: Ensure subnet is associated with NSG'\n  guidelines: \"Every subnet should be associated with NSG for controlling access to\\\n    \\ \\nresources within the subnet.\\n\"\n  category: networking\n  severity: high\nscope:\n  provider: azure\ndefinition:\n  and:\n  - cond_type: connection\n    resource_types:\n    - azurerm_subnet_network_security_group_association\n    connected_resource_types:\n    - azurerm_subnet\n    - azurerm_network_security_group\n    operator: exists\n  - cond_type: filter\n    attribute: resource_type\n    value:\n    - azurerm_subnet\n    operator: within\n"
                        },
                        "recommendation": ""
                    }
                ]
            },
            "recommendation": "",
            "cloudType": "azure",
            "labels": [
                "CODE",
                "KARTIK"
            ],
            "enabled": True,
            "createdOn": 1653003961960,
            "createdBy": "User",
            "lastModifiedOn": 1653003961960,
            "lastModifiedBy": "User",
            "ruleLastModifiedOn": 1653003961960,
            "deleted": False,
            "owner": "Tenant Name",
            "policyMode": "custom",
            "policyCategory": "risk",
            "policyClass": "exposure",
            "remediable": False
        },
        {
            "policyId": "e4080750-1b9a-4b7b-9acf-7057c61eaa9d",
            "name": "Check that all encrypted RDS clusters are tagged with encrypted: True",
            "policyType": "config",
            "policySubTypes": [
                "build"
            ],
            "systemDefault": False,
            "description": "Check that all encrypted RDS clusters are tagged with encrypted: True",
            "severity": "high",
            "rule": {
                "name": "Check that all encrypted RDS clusters are tagged with encrypted: True",
                "parameters": {
                    "withIac": "true",
                    "savedSearch": "false"
                },
                "type": "Config",
                "children": [
                    {
                        "type": "build",
                        "metadata": {
                            "code": "---\nmetadata:\n name: \"Check that all encrypted RDS clusters are tagged with encrypted: True\"\n guidelines: \"Tags Governance - in case of the matched condition below -> add/modify a tag of encrypted:true\"\n category: \"secrets\"\n severity: \"high\"\nscope:\n  provider: \"aws\"\ndefinition:\n and:\n     - cond_type: \"attribute\"\n       resource_types:\n       - \"aws_rds_cluster\"\n       attribute: \"tags.encrypted\"\n       operator: \"equals\"\n       value: \"true\"\n     - or:\n         - cond_type: \"attribute\"\n           resource_types:\n           - \"aws_rds_cluster\"\n           attribute: \"kms_key_id\"\n           operator: \"exists\"\n         - cond_type: \"attribute\"\n           resource_types:\n           - \"aws_rds_cluster\"\n           attribute: \"storage_encrypted\"\n           operator: \"equals\"\n           value: \"true\""
                        },
                        "recommendation": ""
                    }
                ]
            },
            "recommendation": "",
            "cloudType": "aws",
            "labels": [
                "CODE",
                "KARTIK"
            ],
            "enabled": True,
            "createdOn": 1653999052972,
            "createdBy": "User",
            "lastModifiedOn": 1653999052972,
            "lastModifiedBy": "User",
            "ruleLastModifiedOn": 1653999052972,
            "deleted": False,
            "owner": "Tenant Name",
            "policyMode": "custom",
            "policyCategory": "risk",
            "policyClass": "exposure",
            "remediable": False
        }
    ]


class TestUrlValidation(unittest.TestCase):
    """Integration tests for Prisma API URL domain validation."""

    def test_setup_bridgecrew_credentials_rejects_invalid_prisma_url(self):
        """setup_bridgecrew_credentials() must reject non-allowlisted Prisma URLs."""
        instance = BcPlatformIntegration()
        with self.assertRaises(SystemExit) as ctx:
            instance.setup_bridgecrew_credentials(
                repo_id="org/repo",
                prisma_api_url="https://example.com/api",
                source=get_source_type('disabled'),
            )
        self.assertEqual(ctx.exception.code, 2)

    def test_setup_bridgecrew_credentials_rejects_invalid_bc_url(self):
        """setup_bridgecrew_credentials() must reject non-allowlisted BC URLs."""
        instance = BcPlatformIntegration()
        with self.assertRaises(SystemExit) as ctx:
            instance.setup_bridgecrew_credentials(
                repo_id="org/repo",
                bc_api_url="https://example.com",
                source=get_source_type('disabled'),
            )
        self.assertEqual(ctx.exception.code, 2)

    def test_setup_bridgecrew_credentials_accepts_valid_prisma_url(self):
        """Legitimate Prisma Cloud URLs must still work."""
        instance = BcPlatformIntegration()
        instance.setup_bridgecrew_credentials(
            repo_id="org/repo",
            prisma_api_url="https://api0.prismacloud.io",
            source=get_source_type('disabled'),
        )
        self.assertEqual(instance.prisma_api_url, "https://api0.prismacloud.io")

    def test_setup_bridgecrew_credentials_accepts_gov_prisma_url(self):
        """FedRAMP / gov Prisma Cloud URLs must still work."""
        instance = BcPlatformIntegration()
        instance.setup_bridgecrew_credentials(
            repo_id="org/repo",
            prisma_api_url="https://api.gov.prismacloud.io",
            source=get_source_type('disabled'),
        )
        self.assertEqual(instance.prisma_api_url, "https://api.gov.prismacloud.io")

    def test_get_auth_token_rejects_invalid_prisma_url(self):
        """get_auth_token() must reject non-allowlisted Prisma URL."""
        instance = BcPlatformIntegration()
        instance.bc_api_key = '00000000-0000-0000-0000-000000000000::dGVzdA=='
        # Bypass normalize_prisma_url by setting the URL directly
        instance.prisma_api_url = 'https://example.com/api'
        instance.setup_http_manager()
        with self.assertRaises(SystemExit) as ctx:
            instance.get_auth_token()
        self.assertEqual(ctx.exception.code, 2)

    @mock.patch.dict(os.environ, {'BC_API_URL': 'https://example.com'})
    def test_clean_rejects_invalid_bc_api_url_env(self):
        """clean() must reject invalid BC_API_URL environment variable."""
        with self.assertRaises(SystemExit) as ctx:
            BcPlatformIntegration()
        self.assertEqual(ctx.exception.code, 2)

    @mock.patch.dict(os.environ, {'PRISMA_API_URL': 'https://example.com'})
    def test_clean_rejects_invalid_prisma_api_url_env(self):
        """clean() must reject invalid PRISMA_API_URL environment variable."""
        with self.assertRaises(SystemExit) as ctx:
            BcPlatformIntegration()
        self.assertEqual(ctx.exception.code, 2)


if __name__ == '__main__':
    unittest.main()



import unittest

from checkov.common.models.enums import CheckResult
from checkov.terraform.checks.resource.aws.MSKClusterLogging import check


class TestMSKClusterLogging(unittest.TestCase):

    def test_failure(self):
        resource_conf = {
            "name": "test-project",
            "logging_info": [
                {
                    "broker_logs": [
                        {
                            "cloudwatch_logs": [
                                {
                                    "enabled": [False],
                                }
                            ],
                        }
                    ],
                }
            ],
        }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_none(self):
        resource_conf = {
            "name": "test-project",
        }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        resource_conf = {
            "name": "test-project",
            "logging_info": [
                {
                    "broker_logs": [
                        {
                            "cloudwatch_logs": [
                                {
                                    "enabled": [True],
                                }
                            ],
                        }
                    ],
                }
            ],
        }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_all(self):
        resource_conf = {
            "name": "test-project",
            "logging_info": [
                {
                    "broker_logs": [
                        {
                            "cloudwatch_logs": [
                                {
                                    "enabled": [True],
                                }
                            ],
                        },
                        {
                            "firehose": [
                                {
                                    "enabled": [True],
                                }
                            ],
                        },
                        {
                            "s3": [
                                {
                                    "enabled": [True],
                                }
                            ],
                        }
                    ],
                }
            ],
        }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_mixed(self):
        resource_conf = {
            "name": "test-project",
            "logging_info": [
                {
                    "broker_logs": [
                        {
                            "cloudwatch_logs": [
                                {
                                    "enabled": [True],
                                }
                            ],
                        },
                        {
                            "firehose": [
                                {
                                    "enabled": [True],
                                }
                            ],
                        },
                        {
                            "s3": [
                                {
                                    "enabled": [False],
                                }
                            ],
                        }
                    ],
                }
            ],
        }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_failure_empty(self):
        resource_conf = {
            "name": "test-project",
            "logging_info": [
                {
                    "broker_logs": [
                        {
                            "cloudwatch_logs": [
                                {
                                }
                            ],
                        }
                    ],
                }
            ],
        }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)


if __name__ == '__main__':
    unittest.main()



from __future__ import annotations

from pathlib import Path

import pytest

from checkov.ansible.runner import Runner
from checkov.runner_filter import RunnerFilter
from tests.common.graph.checks.test_yaml_policies_base import load_yaml_data

BASE_DIR = Path(__file__).parent
CHECK_ID_MAP: "dict[str, str]" = {}  # will be filled via setup()


# Builtin module checks
def test_GetUrlValidateCerts():
    run_check(base_path=BASE_DIR / "task/builtin", check="GetUrlValidateCerts")


def test_UriValidateCerts():
    run_check(base_path=BASE_DIR / "task/builtin", check="UriValidateCerts")


def test_YumSslVerify():
    run_check(base_path=BASE_DIR / "task/builtin", check="YumSslVerify")


def test_YumValidateCerts():
    run_check(base_path=BASE_DIR / "task/builtin", check="YumValidateCerts")


def test_AptAllowUnauthenticated():
    run_check(base_path=BASE_DIR / "task/builtin", check="AptAllowUnauthenticated")


def test_AptForce():
    run_check(base_path=BASE_DIR / "task/builtin", check="AptForce")


# AWS module checks
def test_EC2EBSOptimized():
    run_check(base_path=BASE_DIR / "task/aws", check="EC2EBSOptimized")


def test_EC2PublicIP():
    run_check(base_path=BASE_DIR / "task/aws", check="EC2PublicIP")


@pytest.fixture(autouse=True)
def setup():
    global CHECK_ID_MAP
    registry = Runner().import_registry()
    CHECK_ID_MAP = {check.__class__.__name__: check.id for entity, check in registry.all_checks()}


def run_check(base_path: Path, check: str) -> None:
    # set path where to find test files
    test_dir_path = base_path / check

    # setup Ansible runner
    runner = Runner()

    # run actual check
    report = runner.run(root_folder=str(test_dir_path), runner_filter=RunnerFilter(checks=CHECK_ID_MAP[check]))

    # get actual results
    summary = report.get_summary()
    passed_checks = {check.resource for check in report.passed_checks}
    failed_checks = {check.resource for check in report.failed_checks}

    # get expected results
    expected = load_yaml_data(dir_path=test_dir_path, source_file_name="expected.yaml")

    # make sure it is a dict
    assert isinstance(expected, dict)

    expected_to_pass = expected.get("pass") or []
    expected_to_fail = expected.get("fail") or []

    # check, if results are correct
    assert summary["passed"] == len(expected_to_pass)
    assert summary["failed"] == len(expected_to_fail)
    assert summary["skipped"] == 0
    assert summary["parsing_errors"] == 0

    assert passed_checks == set(expected_to_pass)
    assert failed_checks == set(expected_to_fail)



import unittest
from pathlib import Path

from checkov.runner_filter import RunnerFilter
from checkov.terraform.checks.resource.kubernetes.CPURequests import check
from checkov.terraform.runner import Runner


class TestCPURequests(unittest.TestCase):
    def test(self):
        # given
        test_files_dir = Path(__file__).parent / "example_CPURequests"

        # when
        report = Runner().run(root_folder=str(test_files_dir), runner_filter=RunnerFilter(checks=[check.id]))

        # then
        summary = report.get_summary()

        passing_resources = {
            "kubernetes_pod.pass",
            "kubernetes_pod_v1.pass",
            "kubernetes_deployment.pass",
            "kubernetes_deployment_v1.pass",
        }

        failing_resources = {
            "kubernetes_pod.fail",
            "kubernetes_pod.fail2",
            "kubernetes_pod.fail3",
            "kubernetes_pod.fail4",
            "kubernetes_pod.fail5",
            "kubernetes_pod_v1.fail",
            "kubernetes_pod_v1.fail2",
            "kubernetes_pod_v1.fail3",
            "kubernetes_pod_v1.fail4",
            "kubernetes_pod_v1.fail5",
            "kubernetes_deployment.fail",
            "kubernetes_deployment.fail2",
            "kubernetes_deployment.fail3",
            "kubernetes_deployment.fail4",
            "kubernetes_deployment.fail5",
            "kubernetes_deployment_v1.fail",
            "kubernetes_deployment_v1.fail2",
            "kubernetes_deployment_v1.fail3",
            "kubernetes_deployment_v1.fail4",
            "kubernetes_deployment_v1.fail5",
        }

        passed_check_resources = {c.resource for c in report.passed_checks}
        failed_check_resources = {c.resource for c in report.failed_checks}

        self.assertEqual(summary["passed"], 2 * 2)
        self.assertEqual(summary["failed"], 18 * 2)
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)

        self.assertEqual(passing_resources, passed_check_resources)
        self.assertEqual(failing_resources, failed_check_resources)


if __name__ == '__main__':
    unittest.main()



from aws_cdk import core
from aws_cdk import aws_appsync as appsync

class AppSyncStack(core.Stack):

    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define the GraphQL API using CfnGraphQLApi
        graphql_api = appsync.CfnGraphQLApi(
            self,
            "AppSyncGraphQLApi",
            name="MyAppSyncAPI",
            authentication_type="API_KEY",  # You can change the authentication type
            log_config=appsync.CfnGraphQLApi.LogConfigProperty(
                exclude_verbose_content=False,
                field_log_level="fieldLogLevel"
            ),
        )


app = core.App()
AppSyncStack(app, "AppSyncStack")
app.synth()



import subprocess

from checkov.helm.runner import Runner


def helm_exists() -> bool:
    try:
        subprocess.run([Runner.helm_command, "version"], check=True, stdout=subprocess.PIPE)
    except Exception:
        return False
    return True



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.MSSQLServerMinTLSVersion import check
from checkov.common.models.enums import CheckResult


class TestMSSQLServerMinTLSVersion(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_mssql_server" "examplea" {
              name                          = var.server_name
              resource_group_name           = var.resource_group.name
              location                      = var.resource_group.location
              version                       = var.sql["version"]
              administrator_login           = var.sql["administrator_login"]
              administrator_login_password  = local.administrator_login_password
              minimum_tls_version           = "1.1"
              public_network_access_enabled = var.sql["public_network_access_enabled"]
              identity {
                type = "SystemAssigned"
              }
              tags = var.common_tags
              }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_mssql_server']['examplea']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_mssql_server" "examplea" {
              name                          = var.server_name
              resource_group_name           = var.resource_group.name
              location                      = var.resource_group.location
              version                       = var.sql["version"]
              administrator_login           = var.sql["administrator_login"]
              administrator_login_password  = local.administrator_login_password
              minimum_tls_version           = "1.2"
              public_network_access_enabled = var.sql["public_network_access_enabled"]
              identity {
                type = "SystemAssigned"
              }
              tags = var.common_tags
              }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_mssql_server']['examplea']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import \
    BaseResourceCheck


class TKEPublicIpAssigned(BaseResourceCheck):
    def __init__(self):
        name = "Ensure Tencent Cloud TKE cluster is not assigned a public IP address"
        id = "CKV_TC_7"
        supported_resources = ['tencentcloud_kubernetes_cluster']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf) -> CheckResult:
        if conf.get("master_config"):
            for idx, mc in enumerate(conf["master_config"]):
                if mc.get("public_ip_assigned") and mc["public_ip_assigned"][0]:
                    self.evaluated_keys = [f"master_config/[{idx}]/public_ip_assigned"]
                    return CheckResult.FAILED
                if mc.get("public_ip_assigned") is None and mc.get("internet_max_bandwidth_out") and mc["internet_max_bandwidth_out"][0] > 0:
                    self.evaluated_keys = [f"master_config/[{idx}]/internet_max_bandwidth_out"]
                    return CheckResult.FAILED

        if conf.get("worker_config"):
            for idx, mc in enumerate(conf["worker_config"]):
                if mc.get("public_ip_assigned") and mc["public_ip_assigned"][0]:
                    self.evaluated_keys = [f"worker_config/[{idx}]/public_ip_assigned"]
                    return CheckResult.FAILED
                if mc.get("public_ip_assigned") is None and mc.get("internet_max_bandwidth_out") and mc["internet_max_bandwidth_out"][0] > 0:
                    self.evaluated_keys = [f"worker_config/[{idx}]/internet_max_bandwidth_out"]
                    return CheckResult.FAILED
        return CheckResult.PASSED


check = TKEPublicIpAssigned()



from __future__ import annotations

from typing import Any

from checkov.arm.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class AzureBatchAccountEndpointAccessDefaultAction(BaseResourceCheck):

    DISABLED_PUBLIC_NETWORK_ACCESS = "disabled"
    FORBIDDEN_NETWORK_ACCESS_DEFAULT_ACTION = "allow"

    def __init__(self) -> None:
        name = "Ensure that if Azure Batch account public network access in case 'enabled' then its account access must be 'deny'"
        id = "CKV_AZURE_248"
        supported_resources = ("Microsoft.Batch/batchAccounts",)
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,)

    @staticmethod
    def _exists_and_lower_equal(actual_value: Any, expected_lowercase_value: str) -> bool:
        return actual_value and str(actual_value).lower() == expected_lowercase_value

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        properties = conf.get('properties')
        if not properties or not isinstance(properties, dict):
            return CheckResult.FAILED

        public_network_access = properties.get('publicNetworkAccess')
        # public network access is disabled, no need to check for account access default action
        if self._exists_and_lower_equal(public_network_access, self.DISABLED_PUBLIC_NETWORK_ACCESS):
            return CheckResult.PASSED

        network_profile = properties.get('networkProfile')
        if not network_profile:
            return CheckResult.PASSED
        account_access = network_profile.get('accountAccess')
        if not account_access:
            return CheckResult.PASSED
        default_action = account_access.get('defaultAction')
        if not self._exists_and_lower_equal(default_action, self.FORBIDDEN_NETWORK_ACCESS_DEFAULT_ACTION):
            return CheckResult.PASSED

        self.evaluated_keys = ["properties/networkProfile/accountAccess/defaultAction"]
        return CheckResult.FAILED


check = AzureBatchAccountEndpointAccessDefaultAction()



import os
import unittest

from checkov.terraform.checks.resource.gcp.GoogleCloudDNSSECEnabled import check
from checkov.common.models.enums import CheckResult
from checkov.terraform.runner import Runner
from checkov.runner_filter import RunnerFilter


class TestCloudDNSSECEnabled(unittest.TestCase):

    def test_failure_no_config(self):
        resource_conf = {"name": ["example-zone"],
                         "dns_name": ["example-de13he3.com."],
                         "description": ["Example DNS zone"]
                         }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_wrong_config(self):
        resource_conf = {"name": ["example-zone"],
                         "dns_name": ["example-de13he3.com."],
                         "description": ["Example DNS zone"],
                         "dnssec_config": [{"state": ["off"]}]
                         }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        resource_conf = {"name": ["example-zone"],
                         "dns_name": ["example-de13he3.com."],
                         "description": ["Example DNS zone"],
                         "dnssec_config": [{"state": ["on"]}]
                         }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_visibility_check(self):
        runner = Runner()
        current_dir = os.path.dirname(os.path.realpath(__file__))

        test_files_dir = current_dir + "/test_GoogleCloudDNSSECEnabled"
        report = runner.run(root_folder=test_files_dir, runner_filter=RunnerFilter(checks=[check.id]))
        summary = report.get_summary()

        self.assertEqual(summary['passed'], 2)
        self.assertEqual(summary['failed'], 3)
        self.assertEqual(summary['skipped'], 0)
        self.assertEqual(summary['parsing_errors'], 0)


if __name__ == '__main__':
    unittest.main()



import unittest

from checkov.common.models.enums import CheckResult
from checkov.terraform.checks.resource.aws.EMRClusterKerberosAttributes import check
import hcl2


class TestEMRClusterKerberosAttributes(unittest.TestCase):

    def test_skipped_no_kerberos(self):
        hcl_res = hcl2.loads("""
resource "aws_emr_cluster" "test" {
  name          = "emr-test-arn"
  release_label = "emr-4.6.0"
  applications  = ["Spark"]

  termination_protection            = false
  keep_job_flow_alive_when_no_steps = true

  ec2_attributes {
    subnet_id                         = aws_subnet.main.id
    emr_managed_master_security_group = aws_security_group.sg.id
    emr_managed_slave_security_group  = aws_security_group.sg.id
    instance_profile                  = aws_iam_instance_profile.emr_profile.arn
  }
}
        """)
        resource_conf = hcl_res['resource'][0]['aws_emr_cluster']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.UNKNOWN, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
resource "aws_emr_cluster" "test" {
  name          = "emr-test-arn"
  release_label = "emr-4.6.0"
  applications  = ["Spark"]

  termination_protection            = false
  keep_job_flow_alive_when_no_steps = true

  kerberos_attributes {
    kdc_admin_password                = "somePassword"  # checkov:skip=CKV_SECRET_6 test secret
    realm                             = "EC2.INTERNAL"
    }
}
        """)
        resource_conf = hcl_res['resource'][0]['aws_emr_cluster']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)
        
    def test_failure(self):
        hcl_res = hcl2.loads("""
resource "aws_emr_cluster" "test" {
  name          = "emr-test-arn"
  release_label = "emr-4.6.0"
  applications  = ["Spark"]

  termination_protection            = false
  keep_job_flow_alive_when_no_steps = true

  kerberos_attributes {
    kdc_admin_password                = "somePassword"
    }
}
        """)
        resource_conf = hcl_res['resource'][0]['aws_emr_cluster']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)           


if __name__ == '__main__':
    unittest.main()



from aws_cdk import core
from aws_cdk import aws_elasticloadbalancingv2 as elbv2

class MyALBStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define ALB with Load Balancer Attributes
        alb = elbv2.CfnLoadBalancer(
            self, 'MyALB',
            name='my-alb',
            type='application',
            load_balancer_attributes=[
                {
                    'key': 'routing.http.drop_invalid_header_fields.enabled',
                    'value': 'false'
                }
            ]
            # Other properties for your ALB
        )

app = core.App()
MyALBStack(app, "MyALBStack")
app.synth()



from aws_cdk import core
from aws_cdk import aws_elasticsearch as elasticsearch

class MyElasticsearchDomainStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define Elasticsearch Domain with Encryption At Rest Enabled
        elasticsearch.CfnDomain(
            self, 'MyElasticsearchDomain',
            domain_name='my-elasticsearch-domain',
            elasticsearch_version='7.10',  # Replace with your desired Elasticsearch version
            encryption_at_rest_options={
                'enabled': True
            }
            # Other properties for your Elasticsearch Domain
        )

app = core.App()
MyElasticsearchDomainStack(app, "MyElasticsearchDomainStack")
app.synth()



from aws_cdk import core
from aws_cdk import aws_ec2 as ec2

class MyEC2InstanceStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define VPC for the EC2 Instance
        vpc = ec2.Vpc(
            self, 'MyVpc',
            max_azs=2  # Replace with the desired number of Availability Zones
        )

        # Define EC2 Instance with Network Interface having Public IP
        instance = ec2.CfnInstance(
            self, 'MyEC2Instance',
            image_id='ami-12345678',  # Replace with your desired AMI ID
            instance_type='t2.micro',  # Replace with your desired instance type
            network_interfaces=[{
                'associate_public_ip_address': True
            }]
            # Other properties for your EC2 Instance
        )

app = core.App()
MyEC2InstanceStack(app, "MyEC2InstanceStack")
app.synth()

class MyEC2LaunchTemplateStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define Launch Template for the EC2 Instance
        launch_template = ec2.CfnLaunchTemplate(
            self, 'MyLaunchTemplate',
            launch_template_name='my-launch-template',
            launch_template_data={
                'network_interfaces': [{
                    'associate_public_ip_address': True
                }]
                # Other properties for your Launch Template Data
            }
        )

app = core.App()
MyEC2LaunchTemplateStack(app, "MyEC2LaunchTemplateStack")
app.synth()



import unittest

import hcl2

from checkov.terraform.checks.resource.gcp.GoogleStorageBucketNotPublic import check
from checkov.common.models.enums import CheckResult


class TestGoogleStorageBucketNotPublic(unittest.TestCase):

    def test_failure_1(self):
        hcl_res = hcl2.loads("""
        resource "google_storage_bucket_iam_member" "member" {
          bucket = google_storage_bucket.default.name
          role = "roles/storage.admin"
          member = "allUsers"
        }
                """)
        resource_conf = hcl_res['resource'][0]['google_storage_bucket_iam_member']['member']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_2(self):
        hcl_res = hcl2.loads("""
        resource "google_storage_bucket_iam_binding" "binding" {
          bucket = google_storage_bucket.default.name
          role = "roles/storage.admin"
          members = [
            "user:jane@example.com",
            "allAuthenticatedUsers"
          ]
        }
                """)
        resource_conf = hcl_res['resource'][0]['google_storage_bucket_iam_binding']['binding']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success_1(self):
        hcl_res = hcl2.loads("""
                resource "google_storage_bucket_iam_member" "member" {
                  bucket = google_storage_bucket.default.name
                  role = "roles/storage.admin"
                  member = "user:jane@example.com"
                }
                        """)
        resource_conf = hcl_res['resource'][0]['google_storage_bucket_iam_member']['member']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_2(self):
        hcl_res = hcl2.loads("""
                resource "google_storage_bucket_iam_binding" "binding" {
                  bucket = google_storage_bucket.default.name
                  role = "roles/storage.admin"
                  members = [
                    "user:jane@example.com"
                  ]
                }
                        """)
        resource_conf = hcl_res['resource'][0]['google_storage_bucket_iam_binding']['binding']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)



if __name__ == '__main__':
    unittest.main()



from aws_cdk import core
from aws_cdk import aws_glue as glue
from aws_cdk import aws_iam as iam

class GlueCrawlerStack(core.Stack):

    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        crawler = glue.CfnCrawler(
            self,
            "MyCrawler",
            name="MyCrawler",
            database_name="mydatabase",
            role=crawler_role.role_arn,
            targets={
                "s3Targets": [
                    {
                        "path": "s3://your-s3-bucket/path/to/crawl",
                    }
                ]
            },
            crawler_security_configuration="aaa"
        )

app = core.App()
GlueCrawlerStack(app, "GlueCrawlerStack")
app.synth()


class GlueDevEndpointStack(core.Stack):

    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Create an AWS Glue Security Configuration (You need to create one separately)
        security_configuration_name = "MySecurityConfiguration"  # Replace with your security config name

        # Create an AWS Glue DevEndpoint
        dev_endpoint = glue.CfnDevEndpoint(
            self,
            "MyDevEndpoint",
            role_arn="arn:aws:iam::YOUR_ACCOUNT_ID:role/YourGlueDevEndpointRole",
            security_configuration=security_configuration_name,
            worker_type="Standard",
            glue_version="1.0",
        )

app = core.App()
GlueDevEndpointStack(app, "GlueDevEndpointStack")
app.synth()

class GlueJobStack(core.Stack):

    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Create an AWS Glue Security Configuration (You need to create one separately)
        security_configuration_name = "MySecurityConfiguration"  # Replace with your security config name

        # Create an AWS Glue Job
        job = glue.CfnJob(
            self,
            "MyGlueJob",
            command={
                "name": "glueetl",
                "pythonVersion": "3"
            },
            default_arguments={
                "--job-language": "python"
            },
            security_configuration=security_configuration_name,
            max_capacity=10,
            glue_version="1.0"
        )

app = core.App()
GlueJobStack(app, "GlueJobStack")
app.synth()




import unittest

import hcl2

from checkov.terraform.checks.resource.azure.AzureBatchAccountUsesKeyVaultEncryption import check
from checkov.common.models.enums import CheckResult


class TestAzureBatchAccountUsesKeyVaultEncryption(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_batch_account" "example" {
              name                 = "testbatchaccount"
              resource_group_name  = azurerm_resource_group.example.name
              location             = azurerm_resource_group.example.location
              pool_allocation_mode = "BatchService"
              storage_account_id   = azurerm_storage_account.example.id
            
              tags = {
                env = "test"
              }
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_batch_account']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
        resource "azurerm_batch_account" "example" {
              name                 = "testbatchaccount"
              resource_group_name  = azurerm_resource_group.example.name
              location             = azurerm_resource_group.example.location
              pool_allocation_mode = "BatchService"
              storage_account_id   = azurerm_storage_account.example.id
              key_vault_reference {
                id = "test"
                url = "https://test.com"
              }
            
              tags = {
                env = "test"
              }
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_batch_account']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



"""Public-key loading. Only ECDSA P-256 + SHA-256 is accepted."""
from __future__ import annotations

import binascii
import hashlib
import logging
from dataclasses import dataclass

# ecdsa ships no py.typed marker; suppress the strict-mode import-untyped
# noise here rather than polluting mypy.ini for a single MR-scoped dependency.
from ecdsa import NIST256p, VerifyingKey  # type: ignore[import-untyped]
from ecdsa.der import UnexpectedDER  # type: ignore[import-untyped]
from ecdsa.errors import MalformedPointError  # type: ignore[import-untyped]

from .errors import SignatureVerificationError


logger = logging.getLogger(__name__)


# Anything outside this set propagates so unrelated bugs surface with their real root cause.
_EXPECTED_PEM_LOAD_ERRORS = (
    UnexpectedDER,
    binascii.Error,
    MalformedPointError,
    ValueError,
    TypeError,
)


@dataclass(frozen=True)
class VerificationKey:
    public_key: VerifyingKey
    fingerprint_hex: str
    source_path: str


def _validate_curve(vk: VerifyingKey, source_path: str) -> VerifyingKey:
    if vk.curve != NIST256p:
        raise SignatureVerificationError(f"unsupported key format in {source_path}")
    return vk


def load_public_keys(paths: "list[str]") -> "list[VerificationKey]":
    keys: "list[VerificationKey]" = []
    for path in paths:
        try:
            with open(path, "rb") as f:
                pem = f.read()
        except OSError as exc:
            raise SignatureVerificationError(
                f"cannot read public key {path}: {exc.strerror or exc}"
            ) from exc

        try:
            vk = VerifyingKey.from_pem(pem)
        except _EXPECTED_PEM_LOAD_ERRORS as exc:
            raise SignatureVerificationError(f"unsupported key format in {path}") from exc

        validated = _validate_curve(vk, path)
        spki_der = validated.to_der()
        fingerprint = hashlib.sha256(spki_der).hexdigest()
        keys.append(VerificationKey(
            public_key=validated,
            fingerprint_hex=fingerprint,
            source_path=path,
        ))
        logger.debug("Loaded verification key %s (fp=%s)", path, fingerprint[:16])

    return keys


__all__ = [
    "VerificationKey",
    "load_public_keys",
]



import os
import unittest
from pathlib import Path
from unittest import mock

from checkov.runner_filter import RunnerFilter
from checkov.terraform.checks.resource.aws.EC2PublicIP import check
from checkov.terraform.runner import Runner


class TestEC2PublicIP(unittest.TestCase):
    @mock.patch.dict(os.environ, {"CHECKOV_ENABLE_FOREACH_HANDLING": "True"})
    def test(self):
        test_files_dir = Path(__file__).parent / "example_EC2PublicIP"
        report = Runner().run(root_folder=str(test_files_dir), runner_filter=RunnerFilter(checks=[check.id]))
        summary = report.get_summary()

        passing_resources = {
            "aws_instance.default",
            "aws_instance.private",
            "aws_launch_template.default",
            "aws_launch_template.private",
            "aws_instance.public_foreach[\"key2\"]",
            "aws_instance.public_foreach_loop_list[\"k\"]",
            "aws_instance.public_foreach_loop_list[\"v\"]",
            "aws_instance.public_foreach_loop_list_of_dicts[\"private\"]",
        }
        failing_resources = {
            "aws_instance.public",
            "aws_launch_template.public",
            "aws_instance.public_foreach[\"key1\"]",
            "aws_instance.public_foreach_loop[\"key3\"]",
            "aws_instance.public_foreach_loop[\"key4\"]",
            "aws_instance.public_foreach_loop_list_of_dicts[\"public\"]",
        }

        passed_check_resources = {c.resource for c in report.passed_checks}
        failed_check_resources = {c.resource for c in report.failed_checks}

        self.assertEqual(summary["passed"], len(passing_resources))
        self.assertEqual(summary["failed"], len(failing_resources))
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)

        self.assertEqual(passing_resources, passed_check_resources)
        self.assertEqual(failing_resources, failed_check_resources)

    @mock.patch.dict(os.environ, {"CHECKOV_ENABLE_MODULES_FOREACH_HANDLING": "True"})
    def test_for_each_poc(self):
        test_files_dir = Path(__file__).parent / "example_EC2PublicIP_foreach"
        report = Runner().run(root_folder=str(test_files_dir), runner_filter=RunnerFilter(checks=[check.id]))
        summary = report.get_summary()

        passing_resources = {
            "module.s3_module[\"a\"].aws_instance.poc_modules_foreach[\"key2\"]",
            "module.s3_module[\"b\"].aws_instance.poc_modules_foreach[\"key2\"]",
        }
        failing_resources = {
            "module.s3_module[\"a\"].aws_instance.poc_modules_foreach[\"key1\"]",
            "module.s3_module[\"b\"].aws_instance.poc_modules_foreach[\"key1\"]",
        }

        passed_check_resources = {c.resource for c in report.passed_checks}
        failed_check_resources = {c.resource for c in report.failed_checks}

        self.assertEqual(summary["passed"], len(passing_resources))
        self.assertEqual(summary["failed"], len(failing_resources))
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)

        self.assertEqual(passing_resources, passed_check_resources)
        self.assertEqual(failing_resources, failed_check_resources)


if __name__ == "__main__":
    unittest.main()



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.CosmosDBDisableAccessKeyWrite import check
from checkov.common.models.enums import CheckResult


class TestCosmosDBDisableAccessKeyWrite(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_cosmosdb_account" "db" {
              name                = "tfex-cosmos-db-${random_integer.ri.result}"
              location            = azurerm_resource_group.rg.location
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_cosmosdb_account']['db']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_cosmosdb_account" "db" {
              name                = "tfex-cosmos-db-${random_integer.ri.result}"
              location            = azurerm_resource_group.rg.location
              access_key_metadata_writes_enabled = false
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_cosmosdb_account']['db']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



from typing import List

from checkov.common.models.enums import CheckCategories
from checkov.arm.base_resource_value_check import BaseResourceValueCheck


class MySQLPublicAccessDisabled(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure 'public network access enabled' is set to 'False' for mySQL servers"
        id = "CKV_AZURE_53"
        supported_resources = ("Microsoft.DBforMySQL/servers", "Microsoft.DBforMySQL/flexibleServers",)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        if self.entity_type == "Microsoft.DBforMySQL/servers":
            return "properties/publicNetworkAccess"
        else:
            return "properties/network/publicNetworkAccess"

    def get_expected_value(self) -> str:
        return "disabled"

    def get_expected_values(self) -> List[str]:
        return ["disabled", "Disabled"]


check = MySQLPublicAccessDisabled()



from checkov.policies_3d.checks_parser import Policy3dParser

def test_3d_policy_parser_init(raw_3d_policy):
    # Act
    parser = Policy3dParser(raw_3d_policy)

    # Assert
    assert parser.schema_version == 'v1'
    assert parser.check_definition == [
      {
        "cves": {
          "or": [
            {
              "and": [
                {
                  "risk_factor": "DoS"
                },
                {
                  "risk_factor": "Medium Severity"
                }
              ]
            }
          ]
        }
      },
      {
        "iac": {
          "or": [
            {
              "violation_id": "BC_K8S_1"
            },
            {
              "violation_id": "BC_K8S_23"
            }
          ]
        }
      }
    ]

def test_3d_policy_parser_parse_check_v1(raw_3d_policy, k8s_record_1, cve_1):
  # Arrange
  parser = Policy3dParser(raw_3d_policy)

  # Act
  check = parser.parse(iac_records=[k8s_record_1], cves_reports=[cve_1])

  # Assert
  assert check
  assert check.id == 'BC_3D_500'
  assert check.category == 'Policy3D'
  assert check.guideline == 'guideline_500'
  assert len(check.predicaments) == 1
  assert len(check.predicaments[0].predicaments[0].predicaments[0].predicates) == 2
  assert len(check.predicaments[0].predicaments[1].predicates) == 2


from __future__ import annotations

import os.path
import sys
import tarfile
import base64
import gzip
import io
import logging
from pathlib import Path
from typing import Dict
from zipfile import ZipFile

from charset_normalizer import from_path

from checkov.common.resource_code_logger_filter import add_resource_code_filter_to_logger

logger = logging.getLogger(__name__)
add_resource_code_filter_to_logger(logger)


def convert_to_unix_path(path: str) -> str:
    return path.replace('\\', '/')


def safe_relpath(file_path: str, root_folder: str | Path | None) -> str:
    """Compute os.path.relpath(file_path, root_folder) safely on Windows.

    On Windows ``os.path.relpath`` raises ``ValueError`` when *file_path* and
    *root_folder* reside on different drives (e.g. ``C:\\`` vs ``D:\\``).
    In that case we fall back to returning just the file basename so that the
    caller still gets a usable (albeit shortened) relative path instead of
    crashing.

    On non-Windows platforms this is a thin wrapper around ``os.path.relpath``.
    """
    try:
        return os.path.relpath(file_path, root_folder)
    except ValueError:
        # Cross-drive on Windows – best-effort: return the basename only
        logger.warning(
            f"Could not compute relative path for '{file_path}' from root '{root_folder}' "
            f"(cross-drive path on Windows); falling back to basename."
        )
        return os.path.basename(file_path)


def _is_within(base: str, target: str) -> bool:
    """Return True iff target resolves inside base (both realpath'd)."""
    base_real = os.path.realpath(base)
    target_real = os.path.realpath(target)
    try:
        return os.path.commonpath([base_real, target_real]) == base_real
    except ValueError:
        return False


def _safe_tar_members(tar: tarfile.TarFile, dest_path: str) -> list[tarfile.TarInfo]:
    """Drop members that would escape dest_path via absolute path, .., or link.

    Used on Python <3.12 where tarfile.data_filter is not available.
    Mirrors PEP 706 'data' filter behavior: reject absolute paths, paths
    resolving outside dest, links whose target resolves outside dest, and
    device/fifo specials.
    """
    safe: list[tarfile.TarInfo] = []
    dest_real = os.path.realpath(dest_path)
    for m in tar.getmembers():
        if m.isdev():
            logger.warning(f"tar: dropping device/special member {m.name!r}")
            continue
        member_path = os.path.join(dest_real, m.name)
        if not _is_within(dest_real, member_path):
            logger.warning(f"tar: dropping out-of-tree member {m.name!r}")
            continue
        if m.islnk() or m.issym():
            link_target = os.path.join(os.path.dirname(member_path), m.linkname)
            if not _is_within(dest_real, link_target):
                logger.warning(f"tar: dropping link {m.name!r} -> {m.linkname!r} (escapes dest)")
                continue
        safe.append(m)
    return safe


def extract_tar_archive(source_path: str, dest_path: str) -> None:
    with tarfile.open(source_path) as tar:
        if sys.version_info >= (3, 12):
            tar.extractall(path=dest_path, filter="data")
        else:
            tar.extractall(path=dest_path, members=_safe_tar_members(tar, dest_path))  # nosec B202


def extract_zip_archive(source_path: str, dest_path: str) -> None:
    with ZipFile(source_path) as zipf:
        dest_real = os.path.realpath(dest_path)
        for name in zipf.namelist():
            target = os.path.join(dest_real, name)
            if not _is_within(dest_real, target):
                logger.warning(f"zip: dropping out-of-tree member {name!r}")
                continue
            zipf.extract(name, path=dest_path)


def compress_file_gzip_base64(input_path: str) -> str:
    try:
        with open(input_path, 'rb') as json_results_file:
            data = json_results_file.read()
        zip_file = gzip.compress(data)  # to gzip - return in bytes
        base64_bytes = base64.b64encode(zip_file)  # to base64
        base64_string = base64_bytes.decode("utf-8")
        return base64_string
    except Exception:
        logging.exception("failed to open and load results file")
        raise


def decompress_file_gzip_base64(compressed_file_body: str) -> bytes:
    try:
        # 1. convert string to bytes
        # 2. decode base64 data
        # 3. wrap decoded binary data with BytesIO to enable reading
        # 4. gunzip compressed data
        base64_bytes = compressed_file_body.encode("utf-8")
        decoded_base64 = base64.b64decode(base64_bytes)
        with gzip.open(io.BytesIO(decoded_base64), 'rb') as file_extracted_body:
            return file_extracted_body.read()
    except Exception:
        logging.exception("failed to extract package file")
        raise


def compress_string_io_tar(string_io: io.StringIO) -> io.BytesIO:
    file_io = io.BytesIO()
    str_data = string_io.getvalue().encode('utf8')
    bio = io.BytesIO(str_data)
    try:
        with tarfile.open(fileobj=file_io, mode='w:gz') as tar:
            info = tar.tarinfo(name='logs_file.txt')
            bio.seek(0)
            info.size = string_io.tell()
            tar.addfile(info, bio)
        file_io.seek(0)
        return file_io
    except Exception:
        logging.exception("failed to compress logging file")
        raise


def compress_multiple_strings_ios_tar(logs_streams: Dict[str, io.StringIO]) -> io.BytesIO:
    tar_stream = io.BytesIO()
    try:
        with tarfile.open(fileobj=tar_stream, mode='w:gz') as tar:
            for filename, stringio in logs_streams.items():
                bytes_io = io.BytesIO(stringio.getvalue().encode())

                tarinfo = tarfile.TarInfo(name=f'{filename}.log')
                tarinfo.size = len(bytes_io.getvalue())

                bytes_io.seek(0)
                tar.addfile(tarinfo, fileobj=bytes_io)

        tar_stream.seek(0)
        return tar_stream
    except Exception:
        logging.exception("failed to compress logging file")
        raise


def read_file_safe(file_path: str) -> str:
    try:
        with open(file_path, 'r') as f:
            file_content = f.read()
            return file_content
    except Exception:
        logging.warning(
            "Could not open file",
            extra={"file_path": file_path}
        )
        return ""


def get_file_size_safe(file_path: str) -> int:
    try:
        return os.path.getsize(file_path)
    except Exception as e:
        logging.warning(
            f"Could not obtain file size, {str(e)}",
            extra={"file_path": file_path}
        )
        return -1


def read_file_with_any_encoding(file_path: str | Path) -> str:
    """Read the file with the system encoding and then try to detect it"""

    file_path = file_path if isinstance(file_path, Path) else Path(file_path)

    try:
        content = file_path.read_text()
    except UnicodeDecodeError:
        logger.info(f"Encoding for file {file_path} is not UTF-8, trying to detect it")
        content = str(from_path(file_path).best())

    return content



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.MonitorLogProfileRetentionDays import check
from checkov.common.models.enums import CheckResult


class TestMonitorLogProfileRetentionDays(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
           resource "azurerm_monitor_log_profile" "example" {
              name = "default"
              categories = [
                "Action",
                "Delete",
                "Write",
              ]
              locations = [
                "westus",
                "global",
              ]
              retention_policy {
                enabled = true
                days    = 7
              }
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_monitor_log_profile']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_1(self):
        hcl_res = hcl2.loads("""
           resource "azurerm_monitor_log_profile" "example" {
              name = "default"
              categories = [
                "Action",
                "Delete",
                "Write",
              ]
              locations = [
                "westus",
                "global",
              ]
              retention_policy {
                enabled = true
                days    = null
              }
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_monitor_log_profile']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success_1(self):
        hcl_res = hcl2.loads("""
           resource "azurerm_monitor_log_profile" "example" {
              name = "default"
              categories = [
                "Action",
                "Delete",
                "Write",
              ]
              locations = [
                "westus",
                "global",
              ]
              retention_policy {
                enabled = true
                days    = 365
              }
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_monitor_log_profile']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_2(self):
        hcl_res = hcl2.loads("""
           resource "azurerm_monitor_log_profile" "example" {
              name = "default"
              categories = [
                "Action",
                "Delete",
                "Write",
              ]
              locations = [
                "westus",
                "global",
              ]
              retention_policy {
                enabled = false
                days    = 0
              }
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_monitor_log_profile']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_failure_missing(self):
        hcl_res = hcl2.loads("""
           resource "azurerm_monitor_log_profile" "example" {
              name = "default"
              categories = [
                "Action",
                "Delete",
                "Write",
              ]
              locations = [
                "westus",
                "global",
              ]
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_monitor_log_profile']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_unknown(self):
        hcl_res = hcl2.loads("""
                   resource "azurerm_monitor_log_profile" "example" {
                      name = "default"
                      categories = [
                        "Action",
                        "Delete",
                        "Write",
                      ]
                      locations = [
                        "westus",
                        "global",
                      ]
                      retention_policy {
                        enabled = true
                        days    = var.days
                      }
                    }
                        """)
        resource_conf = hcl_res['resource'][0]['azurerm_monitor_log_profile']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.UNKNOWN, scan_result)


if __name__ == '__main__':
    unittest.main()



from pathlib import Path

from checkov.common.graph.db_connectors.networkx.networkx_db_connector import NetworkxConnector
from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.graph.graph_builder.graph_components.block_types import BlockType
from checkov.dockerfile.graph_builder.graph_components.resource_types import ResourceType
from checkov.dockerfile.graph_manager import DockerfileGraphManager
from checkov.dockerfile.utils import get_scannable_file_paths, get_files_definitions

RESOURCES_DIR = Path(__file__).parent / "resources"


def test_build_graph_from_definitions():
    # given
    test_dir_path = RESOURCES_DIR / "expose_port"
    test_file = str(test_dir_path / "pass/Dockerfile")
    files_list = get_scannable_file_paths(root_folder=test_dir_path)
    definitions, _ = get_files_definitions(files_list)

    graph_manager = DockerfileGraphManager(db_connector=NetworkxConnector())

    # when
    local_graph = graph_manager.build_graph_from_definitions(definitions=definitions)

    # then
    assert len(local_graph.vertices) == 16
    assert len(local_graph.edges) == 0

    expose_idx = local_graph.vertices_by_path_and_name[(test_file, ResourceType.EXPOSE)]
    expose = local_graph.vertices[expose_idx]

    assert expose.block_type == BlockType.RESOURCE
    assert expose.id == ResourceType.EXPOSE
    assert expose.source == "Dockerfile"
    assert expose.attributes[CustomAttributes.RESOURCE_TYPE] == ResourceType.EXPOSE
    assert expose.config == {
        "content": "EXPOSE 3000 80 443\n",
        "value": "3000 80 443",
        "__startline__": 2,
        "__endline__": 2,
    }



from aws_cdk import core
from aws_cdk import aws_apigatewayv2 as apigatewayv2

class MyApiGatewayV2StageStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define API Gateway V2 Stage with AccessLogSettings/DestinationArn set
        api_stage = apigatewayv2.CfnStage(
            self, 'MyApiGatewayV2Stage',
            api_id='api_id_here',  # Replace with your API ID
            stage_name='myStage',
            access_log_settings=apigatewayv2.CfnStage.AccessLogSettingsProperty(
                destination_arn='arn:aws:logs:REGION:ACCOUNT_ID:log-group:LOG_GROUP_NAME'
                # Replace with the actual DestinationArn value
            )
            # Add other properties as needed for your stage
        )

app = core.App()
MyApiGatewayV2StageStack(app, "MyApiGatewayV2StageStack")
app.synth()

from aws_cdk import core
from aws_cdk import aws_apigatewayv2 as apigatewayv2

class MyServerlessHttpApiStack2(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define a Serverless HTTP API with access log settings
        serverless_api = apigatewayv2.CfnApi(
            self, 'MyServerlessHttpApi',
            name='MyHTTPAPI',
            protocol_type='HTTP',
            access_log_settings=apigatewayv2.CfnApi.AccessLogSettingsProperty(
                destination_arn='arn:aws:logs:REGION:ACCOUNT_ID:log-group:LOG_GROUP_NAME'
                # Replace with the actual DestinationArn value
            )
            # Add other properties as needed for your HTTP API
        )

app = core.App()
MyServerlessHttpApiStack2(app, "MyServerlessHttpApiStack2")
app.synth()



import os
import unittest

from checkov.cloudformation.checks.resource.aws.ALBListenerTLS12 import check
from checkov.cloudformation.runner import Runner
from checkov.runner_filter import RunnerFilter


class TestALBListenerTLS12(unittest.TestCase):

    def test_summary(self):
        runner = Runner()
        current_dir = os.path.dirname(os.path.realpath(__file__))

        test_files_dir = current_dir + "/example_ALBListenerTLS12"
        report = runner.run(root_folder=test_files_dir,runner_filter=RunnerFilter(checks=[check.id]))
        summary = report.get_summary()

        passing_resources = {
            'AWS::ElasticLoadBalancingV2::Listener.ListenerHTTPSPASSED1',
            'AWS::ElasticLoadBalancingV2::Listener.ListenerHTTPPASSED2',
            'AWS::ElasticLoadBalancingV2::Listener.ListenerTLSPASSED1',
            'AWS::ElasticLoadBalancingV2::Listener.ListenerTLSPASSED2',
            'AWS::ElasticLoadBalancingV2::Listener.ListenerTLSPASSED3',
            'AWS::ElasticLoadBalancingV2::Listener.ListenerTCPPASSED4',
            'AWS::ElasticLoadBalancingV2::Listener.ListenerHTTPSPASS13'
        }

        failing_resources = {
            'AWS::ElasticLoadBalancingV2::Listener.ListenerHTTPSFAILED1',
            'AWS::ElasticLoadBalancingV2::Listener.ListenerHTTPSFAILED2',
            'AWS::ElasticLoadBalancingV2::Listener.ListenerTLSFAILED1',
            'AWS::ElasticLoadBalancingV2::Listener.ListenerTLSFAILED2',
            'AWS::ElasticLoadBalancingV2::Listener.ListenerTLSFAILED3',
            'AWS::ElasticLoadBalancingV2::Listener.ListenerHTTPFAILED1'
        }

        unknown_resource = 'AWS::ElasticLoadBalancingV2::Listener.ListenerHTTPUnknown'
        passed_check_resources = set([c.resource for c in report.passed_checks])
        failed_check_resources = set([c.resource for c in report.failed_checks])

        self.assertEqual(summary['passed'], passing_resources.__len__())
        self.assertEqual(summary['failed'], failing_resources.__len__())
        self.assertEqual(summary['skipped'], 0)
        self.assertEqual(summary['parsing_errors'], 0)
        self.assertNotIn(unknown_resource, passed_check_resources)
        self.assertNotIn(unknown_resource, failed_check_resources)
        self.assertEqual(passing_resources, passed_check_resources)
        self.assertEqual(failing_resources, failed_check_resources)

if __name__ == '__main__':
    unittest.main()



import unittest

import hcl2

from checkov.terraform.checks.provider.bridgecrew.credentials import check
from checkov.common.models.enums import CheckResult


class TestCredentials(unittest.TestCase):
    def test_success(self):
        hcl_res = hcl2.loads(
            """
            provider "bridgecrew" {}
            """
        )
        provider_conf = hcl_res["provider"][0]["bridgecrew"]
        scan_result = check.scan_provider_conf(conf=provider_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_failure(self):
        hcl_res = hcl2.loads(
            """
            provider "bridgecrew" {
                token = "80e54890-f282-4595-ab3d-45f9bd874987"
            }
            """
        )
        provider_conf = hcl_res["provider"][0]["bridgecrew"]
        scan_result = check.scan_provider_conf(conf=provider_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)


if __name__ == "__main__":
    unittest.main()



import os
from contextlib import ExitStack as does_not_raise
from pathlib import Path
from unittest import mock

import pytest

from checkov.common.util.consts import DEFAULT_EXTERNAL_MODULES_DIR
from checkov.common.util.env_vars_config import env_vars_config
from checkov.terraform.module_loading.loaders.bitbucket_loader import BitbucketLoader # noqa
from checkov.terraform.module_loading.loaders.git_loader import GenericGitLoader # noqa
from checkov.terraform.module_loading.loaders.github_loader import GithubLoader # noqa
from checkov.terraform.module_loading.module_params import ModuleParams
from checkov.terraform.module_loading.registry import ModuleLoaderRegistry # noqa
from checkov.terraform.module_loading.content import ModuleContent
from checkov.terraform.module_loading.loaders.github_access_token_loader import GithubAccessTokenLoader # noqa
from checkov.terraform.module_loading.loaders.bitbucket_access_token_loader import BitbucketAccessTokenLoader # noqa


@pytest.mark.parametrize(
    "source, source_version, expected_content_path, expected_git_url, expected_dest_dir, expected_module_source, expected_inner_module",
    [
        (
            "terraform-aws-modules/security-group/aws",
            "4.0.0",
            "github.com/terraform-aws-modules/terraform-aws-security-group/ff2efb814c924572d27280b99a799fc34d061109",
            "https://github.com/terraform-aws-modules/terraform-aws-security-group?ref=ff2efb814c924572d27280b99a799fc34d061109",
            "github.com/terraform-aws-modules/terraform-aws-security-group/v4.0.0",
            "git::https://github.com/terraform-aws-modules/terraform-aws-security-group?ref=ff2efb814c924572d27280b99a799fc34d061109",
            "",
        ),
        (
            "terraform-aws-modules/security-group/aws//modules/http-80",
            "4.0.0",
            "github.com/terraform-aws-modules/terraform-aws-security-group/ff2efb814c924572d27280b99a799fc34d061109/modules/http-80",
            "https://github.com/terraform-aws-modules/terraform-aws-security-group?ref=ff2efb814c924572d27280b99a799fc34d061109",
            "github.com/terraform-aws-modules/terraform-aws-security-group/v4.0.0",
            "git::https://github.com/terraform-aws-modules/terraform-aws-security-group?ref=ff2efb814c924572d27280b99a799fc34d061109",
            "modules/http-80",
        )
    ],
    ids=["module_with_version", "inner_module_with_version"],
)
@mock.patch("checkov.terraform.module_loading.loaders.git_loader.GitGetter", autospec=True)
def test_load_terraform_registry(
    git_getter,
    tmp_path: Path,
    source,
    source_version,
    expected_content_path,
    expected_git_url,
    expected_dest_dir,
    expected_module_source,
    expected_inner_module,
):
    # given
    current_dir = tmp_path / "tf_registry"
    registry = ModuleLoaderRegistry(download_external_modules=True)
    registry.module_content_cache = {}

    # when
    content = registry.load(current_dir=str(current_dir), source=source, source_version=source_version)

    # then
    assert content.loaded()
    assert content.path() == str(Path(DEFAULT_EXTERNAL_MODULES_DIR) / expected_content_path)

    git_getter.assert_called_once_with(expected_git_url, mock.ANY)


@pytest.mark.parametrize(
    "source, expected_content_path, expected_git_url, expected_dest_dir, expected_module_source, expected_inner_module",
    [
        (
            "git::https://example.com/network.git",
            "example.com/network/HEAD",
            "https://example.com/network.git",
            "example.com/network/HEAD",
            "git::https://example.com/network.git",
            "",
        ),
        (
            "git::https://example.com/network.git?ref=v1.2.0",
            "example.com/network/v1.2.0",
            "https://example.com/network.git?ref=v1.2.0",
            "example.com/network/v1.2.0",
            "git::https://example.com/network.git?ref=v1.2.0",
            "",
        ),
        (
            "git::https://example.com/network.git//modules/vpc",
            "example.com/network/HEAD/modules/vpc",
            "https://example.com/network",
            "example.com/network/HEAD",
            "git::https://example.com/network",
            "modules/vpc",
        ),
        (
            "git::https://example.com/network.git//modules/vpc?ref=v1.2.0",
            "example.com/network/v1.2.0/modules/vpc",
            "https://example.com/network?ref=v1.2.0",
            "example.com/network/v1.2.0",
            "git::https://example.com/network?ref=v1.2.0",
            "modules/vpc",
        ),
        (
            "git::ssh://username@example.com/network.git",
            "example.com/network/HEAD",
            "ssh://username@example.com/network.git",
            "example.com/network/HEAD",
            "git::ssh://username@example.com/network.git",
            "",
        ),
        (
            "git::ssh://username@example.com/network.git?ref=v1.2.0",
            "example.com/network/v1.2.0",
            "ssh://username@example.com/network.git?ref=v1.2.0",
            "example.com/network/v1.2.0",
            "git::ssh://username@example.com/network.git?ref=v1.2.0",
            "",
        ),
        (
            "git::username@example.com/network.git",
            "example.com/network/HEAD",
            "username@example.com/network.git",
            "example.com/network/HEAD",
            "git::username@example.com/network.git",
            "",
        ),
        (
            "git::username@example.com/network.git?ref=v1.2.0",
            "example.com/network/v1.2.0",
            "username@example.com/network.git?ref=v1.2.0",
            "example.com/network/v1.2.0",
            "git::username@example.com/network.git?ref=v1.2.0",
            "",
        ),
        (
            "git::ssh://git@github.com/bridgecrewio/terragoat//modules/s3-encrypted",
            "git@github.com/bridgecrewio/terragoat/HEAD/modules/s3-encrypted",
            "ssh://git@github.com/bridgecrewio/terragoat",
            "git@github.com/bridgecrewio/terragoat/HEAD",
            "git::ssh://git@github.com/bridgecrewio/terragoat",
            "modules/s3-encrypted",
        ),
        (
            "git::git@github.com/bridgecrewio/terragoat//modules/s3-encrypted",
            "git@github.com/bridgecrewio/terragoat/HEAD/modules/s3-encrypted",
            "ssh://git@github.com/bridgecrewio/terragoat",
            "git@github.com/bridgecrewio/terragoat/HEAD",
            "git::ssh://git@github.com/bridgecrewio/terragoat",
            "modules/s3-encrypted",
        ),
    ],
    ids=[
        "module",
        "module_with_version",
        "inner_module",
        "inner_module_with_version",
        "module_over_ssh",
        "module_over_ssh_with_version",
        "module_over_ssh_without_protocol",
        "module_over_ssh_without_protocol_with_version",
        "git_username",
        "git::git@ syntax"
    ],
)
@mock.patch("checkov.terraform.module_loading.loaders.git_loader.GitGetter", autospec=True)
def test_load_generic_git(
    git_getter,
    tmp_path: Path,
    source,
    expected_content_path,
    expected_git_url,
    expected_dest_dir,
    expected_module_source,
    expected_inner_module,
):
    # given
    current_dir = tmp_path / "generic"
    registry = ModuleLoaderRegistry(download_external_modules=True)
    registry.module_content_cache = {}

    # when
    content = registry.load(current_dir=str(current_dir), source=source, source_version="latest")

    # then
    assert content.loaded()
    assert content.path() == str(Path(DEFAULT_EXTERNAL_MODULES_DIR) / expected_content_path)

    git_getter.assert_called_once_with(expected_git_url, mock.ANY)


@pytest.mark.parametrize(
    "source, expected_content_path, expected_git_url, expected_dest_dir, expected_module_source, expected_inner_module",
    [
        (
            "github.com/terraform-aws-modules/terraform-aws-security-group",
            "github.com/terraform-aws-modules/terraform-aws-security-group/HEAD",
            "https://github.com/terraform-aws-modules/terraform-aws-security-group",
            "github.com/terraform-aws-modules/terraform-aws-security-group/HEAD",
            "git::https://github.com/terraform-aws-modules/terraform-aws-security-group",
            "",
        ),
        (
            "github.com/terraform-aws-modules/terraform-aws-security-group?ref=v4.0.0",
            "github.com/terraform-aws-modules/terraform-aws-security-group/v4.0.0",
            "https://github.com/terraform-aws-modules/terraform-aws-security-group?ref=v4.0.0",
            "github.com/terraform-aws-modules/terraform-aws-security-group/v4.0.0",
            "git::https://github.com/terraform-aws-modules/terraform-aws-security-group?ref=v4.0.0",
            "",
        ),
        (
            "github.com/terraform-aws-modules/terraform-aws-security-group//modules/http-80",
            "github.com/terraform-aws-modules/terraform-aws-security-group/HEAD/modules/http-80",
            "https://github.com/terraform-aws-modules/terraform-aws-security-group",
            "github.com/terraform-aws-modules/terraform-aws-security-group/HEAD",
            "git::https://github.com/terraform-aws-modules/terraform-aws-security-group",
            "modules/http-80",
        ),
        (
            "github.com/terraform-aws-modules/terraform-aws-security-group//modules/http-80?ref=v4.0.0",
            "github.com/terraform-aws-modules/terraform-aws-security-group/v4.0.0/modules/http-80",
            "https://github.com/terraform-aws-modules/terraform-aws-security-group?ref=v4.0.0",
            "github.com/terraform-aws-modules/terraform-aws-security-group/v4.0.0",
            "git::https://github.com/terraform-aws-modules/terraform-aws-security-group?ref=v4.0.0",
            "modules/http-80",
        ),
    ],
    ids=["module", "module_with_version", "inner_module", "inner_module_with_version"],
)
@mock.patch("checkov.terraform.module_loading.loaders.git_loader.GitGetter", autospec=True)
def test_load_github(
    git_getter,
    tmp_path: Path,
    source,
    expected_content_path,
    expected_git_url,
    expected_dest_dir,
    expected_module_source,
    expected_inner_module,
):
    # given
    current_dir = tmp_path / "github"
    registry = ModuleLoaderRegistry(download_external_modules=True)
    registry.module_content_cache = {}

    # when
    content = registry.load(current_dir=str(current_dir), source=source, source_version="latest")

    # then
    assert content.loaded()
    assert content.path() == str(Path(DEFAULT_EXTERNAL_MODULES_DIR) / expected_content_path)

    git_getter.assert_called_once_with(expected_git_url, mock.ANY)


# TODO: create a dummy repo in bitbucket for more consistent tests
@pytest.mark.parametrize(
    "source, expected_content_path, expected_git_url, expected_dest_dir, expected_module_source, expected_inner_module",
    [
        (
            "bitbucket.org/nuarch/terraform-aws-rancher-server-ha",
            "bitbucket.org/nuarch/terraform-aws-rancher-server-ha/HEAD",
            "https://bitbucket.org/nuarch/terraform-aws-rancher-server-ha",
            "bitbucket.org/nuarch/terraform-aws-rancher-server-ha/HEAD",
            "git::https://bitbucket.org/nuarch/terraform-aws-rancher-server-ha",
            "",
        ),
        (
            "bitbucket.org/nuarch/terraform-aws-rancher-server-ha?ref=v0.1.0",
            "bitbucket.org/nuarch/terraform-aws-rancher-server-ha/v0.1.0",
            "https://bitbucket.org/nuarch/terraform-aws-rancher-server-ha?ref=v0.1.0",
            "bitbucket.org/nuarch/terraform-aws-rancher-server-ha/v0.1.0",
            "git::https://bitbucket.org/nuarch/terraform-aws-rancher-server-ha?ref=v0.1.0",
            "",
        ),
        (
            "bitbucket.org/nuarch/terraform-aws-rancher-server-ha//rancher2-ha",
            "bitbucket.org/nuarch/terraform-aws-rancher-server-ha/HEAD/rancher2-ha",
            "https://bitbucket.org/nuarch/terraform-aws-rancher-server-ha",
            "bitbucket.org/nuarch/terraform-aws-rancher-server-ha/HEAD",
            "git::https://bitbucket.org/nuarch/terraform-aws-rancher-server-ha",
            "rancher2-ha",
        ),
        (
            "bitbucket.org/nuarch/terraform-aws-rancher-server-ha//rancher2-ha?ref=v0.1.0",
            "bitbucket.org/nuarch/terraform-aws-rancher-server-ha/v0.1.0/rancher2-ha",
            "https://bitbucket.org/nuarch/terraform-aws-rancher-server-ha?ref=v0.1.0",
            "bitbucket.org/nuarch/terraform-aws-rancher-server-ha/v0.1.0",
            "git::https://bitbucket.org/nuarch/terraform-aws-rancher-server-ha?ref=v0.1.0",
            "rancher2-ha",
        ),
    ],
    ids=["module", "module_with_version", "inner_module", "inner_module_with_version"],
)
@mock.patch("checkov.terraform.module_loading.loaders.git_loader.GitGetter", autospec=True)
def test_load_bitbucket(
    git_getter,
    tmp_path: Path,
    source,
    expected_content_path,
    expected_git_url,
    expected_dest_dir,
    expected_module_source,
    expected_inner_module,
):
    # given
    current_dir = tmp_path / "bitbucket"
    registry = ModuleLoaderRegistry(download_external_modules=True)
    registry.module_content_cache = {}

    # when
    content = registry.load(current_dir=str(current_dir), source=source, source_version="latest")

    # then
    assert content.loaded()
    assert content.path() == str(Path(DEFAULT_EXTERNAL_MODULES_DIR) / expected_content_path)

    git_getter.assert_called_once_with(expected_git_url, mock.ANY)


@pytest.mark.parametrize(
    "source, expected_content_path, expected_exception",
    [
        ("./loaders/resources", "loaders/resources", does_not_raise()),
        ("../module_loading/loaders/resources", "loaders/resources", does_not_raise()),
        ("./does_not_exist", "", pytest.raises(FileNotFoundError)),
    ],
    ids=["current_dir", "parent_dir", "not_exists"],
)
@mock.patch("checkov.terraform.module_loading.loaders.git_loader.GitGetter", autospec=True)
def test_load_local_path(git_getter, tmp_path: Path, source, expected_content_path, expected_exception):
    # given
    current_dir = Path(__file__).parent
    registry = ModuleLoaderRegistry()
    registry.module_content_cache = {}

    # when
    with expected_exception:
        content = registry.load(current_dir=str(current_dir), source=source, source_version="latest")

        # then
        assert content.loaded()
        assert content.path() == str(current_dir / expected_content_path)

        git_getter.assert_not_called()


@pytest.mark.parametrize(
    "source, expected_content_path, expected_git_url, expected_dest_dir, expected_module_source, expected_inner_module",
    [
        (
            "github.com/kartikp10/terraform-aws-s3-bucket1",
            "github.com/kartikp10/terraform-aws-s3-bucket1/HEAD",
            "https://x-access-token:ghp_xxxxxxxxxxxxxxxxx@github.com/kartikp10/terraform-aws-s3-bucket1",  # checkov:skip=CKV_SECRET_4 test secret
            "github.com/kartikp10/terraform-aws-s3-bucket1/HEAD",
            "git::https://x-access-token:ghp_xxxxxxxxxxxxxxxxx@github.com/kartikp10/terraform-aws-s3-bucket1",
            "",
        ),
       (
            "git::https://github.com/kartikp10/terraform-aws-s3-bucket1.git",
            "github.com/kartikp10/terraform-aws-s3-bucket1/HEAD",
            "https://x-access-token:ghp_xxxxxxxxxxxxxxxxx@github.com/kartikp10/terraform-aws-s3-bucket1.git",
            "github.com/kartikp10/terraform-aws-s3-bucket1/HEAD",
            "git::https://x-access-token:ghp_xxxxxxxxxxxxxxxxx@github.com/kartikp10/terraform-aws-s3-bucket1.git",
            "",
        ),
       (
           "git@github.com:kartikp10/terraform-aws-s3-bucket1.git",
            "github.com/kartikp10/terraform-aws-s3-bucket1/HEAD",
            "https://x-access-token:ghp_xxxxxxxxxxxxxxxxx@github.com/kartikp10/terraform-aws-s3-bucket1.git",
            "github.com/kartikp10/terraform-aws-s3-bucket1/HEAD",
            "git::https://x-access-token:ghp_xxxxxxxxxxxxxxxxx@github.com/kartikp10/terraform-aws-s3-bucket1.git",
            "",
        ),
       (
           "git::ssh://git@github.com/kartikp10/terraform-aws-s3-bucket1.git",
            "github.com/kartikp10/terraform-aws-s3-bucket1/HEAD",
            "https://x-access-token:ghp_xxxxxxxxxxxxxxxxx@github.com/kartikp10/terraform-aws-s3-bucket1.git",
            "github.com/kartikp10/terraform-aws-s3-bucket1/HEAD",
            "git::https://x-access-token:ghp_xxxxxxxxxxxxxxxxx@github.com/kartikp10/terraform-aws-s3-bucket1.git",
            "",
        ),
        (
            "github.com/kartikp10/terraform-aws-security-group//modules/http-80",
            "github.com/kartikp10/terraform-aws-security-group/HEAD/modules/http-80",
            "https://x-access-token:ghp_xxxxxxxxxxxxxxxxx@github.com/kartikp10/terraform-aws-security-group",
            "github.com/kartikp10/terraform-aws-security-group/HEAD",
            "git::https://x-access-token:ghp_xxxxxxxxxxxxxxxxx@github.com/kartikp10/terraform-aws-security-group",
            "modules/http-80",
        ),
        (
            "git::ssh://git@github.com/kartikp10/terraform-aws-s3-bucket1.git?ref=v1.2.0",
            "github.com/kartikp10/terraform-aws-s3-bucket1/v1.2.0",
            "https://x-access-token:ghp_xxxxxxxxxxxxxxxxx@github.com/kartikp10/terraform-aws-s3-bucket1.git?ref=v1.2.0",
            "github.com/kartikp10/terraform-aws-s3-bucket1/v1.2.0",
            "git::https://x-access-token:ghp_xxxxxxxxxxxxxxxxx@github.com/kartikp10/terraform-aws-s3-bucket1.git?ref=v1.2.0",
            "",
        ),
       (
           "git@github.com:kartikp10/terraform-aws-security-group.git//modules/http-80",
            "github.com/kartikp10/terraform-aws-security-group/HEAD",
            "https://x-access-token:ghp_xxxxxxxxxxxxxxxxx@github.com/kartikp10/terraform-aws-security-group",
            "github.com/kartikp10/terraform-aws-security-group/HEAD",
            "git::https://x-access-token:ghp_xxxxxxxxxxxxxxxxx@github.com/kartikp10/terraform-aws-security-group",
            "modules/http-80",
        )
    ],
    ids=["github_http_module", "generic_git_module", "ssh_github_module", "generic_ssh_module","github_http_module", "generic_ssh_module_version", "github_ssh_module_version"],
)
@mock.patch.dict(os.environ, {"GITHUB_PAT": "ghp_xxxxxxxxxxxxxxxxx"})
@mock.patch("checkov.terraform.module_loading.loaders.git_loader.GitGetter", autospec=True)
def test_load_github_private(
    git_getter,
    tmp_path: Path,
    source,
    expected_content_path,
    expected_git_url,
    expected_dest_dir,
    expected_module_source,
    expected_inner_module,
):
    git_getter.side_effect = [Exception(), None]
    # given
    current_dir = tmp_path / "github_private"
    registry = ModuleLoaderRegistry(download_external_modules=True)
    registry.module_content_cache = {}

    # when
    registry.loaders = [GithubAccessTokenLoader()]
    registry.load(current_dir=str(current_dir), source=source, source_version="latest")

    # then
    git_getter.assert_called_with(expected_git_url, create_clone_and_result_dirs=False)


@pytest.mark.parametrize(
    "source, expected_content_path, expected_git_url, expected_dest_dir, expected_module_source, expected_inner_module",
    [
        (
            "bitbucket.org/kartikp10/terraform-aws-s3-bucket1",
            "bitbucket.org/kartikp10/terraform-aws-s3-bucket1/HEAD",
            "https://x-token-auth:xxxxxxxxxxxxxxxxx@bitbucket.org/kartikp10/terraform-aws-s3-bucket1",  # checkov:skip=CKV_SECRET_4 test secret
            "bitbucket.org/kartikp10/terraform-aws-s3-bucket1/HEAD",
            "git::https://x-token-auth:xxxxxxxxxxxxxxxxx@bitbucket.org/kartikp10/terraform-aws-s3-bucket1",
            "",
        )
    ],
    ids=["module"],
)
@mock.patch.dict(os.environ, {"BITBUCKET_TOKEN": "xxxxxxxxxxxxxxxxx"})  # checkov:skip=CKV_SECRET_6 test secret
@mock.patch("checkov.terraform.module_loading.loaders.git_loader.GitGetter", autospec=True)
def test_load_bitbucket_private(
    git_getter,
    tmp_path: Path,
    source,
    expected_content_path,
    expected_git_url,
    expected_dest_dir,
    expected_module_source,
    expected_inner_module,
):
    git_getter.side_effect = [Exception(), None]
    # given
    current_dir = tmp_path / "bitbucket_private"
    registry = ModuleLoaderRegistry(download_external_modules=True)
    registry.module_content_cache = {}

    # when
    registry.loaders = [BitbucketAccessTokenLoader()]
    registry.load(current_dir=str(current_dir), source=source, source_version="latest")

    # then
    git_getter.assert_called_with(expected_git_url, create_clone_and_result_dirs=False)


def test_load_terraform_registry_with_real_download(tmp_path: Path):
    # given
    current_dir = str(tmp_path / "tf_download")
    registry = ModuleLoaderRegistry(download_external_modules=True, external_modules_folder_name=DEFAULT_EXTERNAL_MODULES_DIR)
    registry.module_content_cache = {}
    registry.root_dir = current_dir

    source = "terraform-aws-modules/security-group/aws"

    # when
    content = registry.load(current_dir=current_dir, source=source, source_version="~> 3.0")

    expected_content_path = os.path.join(
        current_dir,
        DEFAULT_EXTERNAL_MODULES_DIR,
        "github.com/terraform-aws-modules/terraform-aws-security-group",
    )

    assert content.loaded()
    content_path = content.path()
    assert content_path.startswith(f"{expected_content_path}/v3.") or \
           content_path.startswith(f"{expected_content_path}/2cd10c8aca557fd858f401616d5c3b27e2a7b595")


def test_load_terraform_registry_check_cache(tmp_path: Path):
    # given
    current_dir = str(tmp_path / "cache_check")
    registry = ModuleLoaderRegistry(download_external_modules=True)
    registry.module_content_cache = {}
    registry.root_dir = current_dir

    source1 = "git::https://github.com/bridgecrewio/checkov_not_working1.git"
    source2 = "git::https://github.com/bridgecrewio/checkov_not_working2.git"

    # when
    registry.load(current_dir=current_dir, source=source1, source_version="latest")

    assert source1 in registry.failed_urls_cache

    registry.load(current_dir=current_dir, source=source2, source_version="latest")

    # then
    assert source1 in registry.failed_urls_cache
    assert source2 in registry.failed_urls_cache


def test_loader_equality():
    githubLoaderOne = GithubLoader()
    githubLoaderTwo = GithubLoader()
    assert githubLoaderOne == githubLoaderTwo
    bitLoader = BitbucketLoader()
    assert githubLoaderOne != bitLoader
    genericLoader = GenericGitLoader()
    assert githubLoaderOne != genericLoader and bitLoader != genericLoader


def test_multiple_similar_loaders():
    registry = ModuleLoaderRegistry(download_external_modules=True)
    assert len(registry.loaders) == 7
    GithubLoader()
    GithubLoader()
    GenericGitLoader()
    BitbucketLoader()
    assert len(registry.loaders) == 7

@mock.patch.object(env_vars_config, 'CHECKOV_EXPERIMENTAL_TERRAFORM_MANAGED_MODULES', True)
def test_latest_tf_managed(tmp_path: Path):
    registry = ModuleLoaderRegistry(download_external_modules=False)
    registry.module_content_cache = {
        'terraform-aws-modules/iam:5.55.0': ModuleContent('xxx')
    }
    registry.module_latest = {
        'terraform-aws-modules/iam': '5.55.0'
    }

    mc = registry.load(str(tmp_path / 'cache_check'), source='terraform-aws-modules/iam', source_version='latest')
    assert mc and mc.path() == 'xxx'

@mock.patch.object(env_vars_config, 'CHECKOV_EXPERIMENTAL_TERRAFORM_MANAGED_MODULES', True)
def test_latest_tf_managed_registry(tmp_path: Path):
    registry = ModuleLoaderRegistry(download_external_modules=False)
    registry.module_content_cache = {
        'registry.terraform.io/terraform-aws-modules/iam:5.55.0': ModuleContent('xxx')
    }
    registry.module_latest = {
        'registry.terraform.io/terraform-aws-modules/iam': '5.55.0'
    }

    mc = registry.load(str(tmp_path / 'cache_check'), source='terraform-aws-modules/iam', source_version='latest')
    assert mc and mc.path() == 'xxx'


def test_github_is_matching_loader(tmp_path: Path):
    loader = GithubLoader()
    dummy_dir = tmp_path.as_posix()

    params = ModuleParams(
        root_dir=dummy_dir,
        current_dir=dummy_dir,
        source="",
        source_version=None,
        dest_dir=dummy_dir,
        external_modules_folder_name=".external_modules"
    )
    loader.discover(params)

    # --- Case 1: github.com/org/repo ---
    params.module_source = "github.com/org/repo"
    assert loader._is_matching_loader(params) is True
    assert params.module_source == "git::https://github.com/org/repo"

    # --- Case 2: git@github.com:org/repo ---
    params.module_source = "git@github.com:org/repo"
    assert loader._is_matching_loader(params) is True
    assert params.module_source == "git::ssh://git@github.com/org/repo"

    # --- Case 3: git::git@github.com:org/repo ---
    params.module_source = "git::git@github.com:org/repo"
    assert loader._is_matching_loader(params) is True
    assert params.module_source == "git::ssh://git@github.com/org/repo"



import logging
import unittest

from checkov.common.logger_streams import LoggerStreams
from checkov.logging_init import log_stream, erase_log_stream

class TestLoggerStreams(unittest.TestCase):
    def test_stream_collect_valid_logs(self) -> None:
        log_message = 'this is a log message for testing'
        stream_name = 'main_stream'
        erase_log_stream()
        logger_streams = LoggerStreams()
        logger_streams.add_stream(stream_name, log_stream)
        logging.error(log_message)

        # validate logs
        stream_content = logger_streams.get_streams().get(stream_name).getvalue()
        self.assertIn(log_message, stream_content)

        # validate eraser of logs
        erase_log_stream()
        stream_content = logger_streams.get_streams().get(stream_name).getvalue()
        self.assertEqual(stream_content, '')


if __name__ == "__main__":
    unittest.main()



import unittest
from pathlib import Path

from checkov.secrets.plugins.entropy_keyword_combinator import EntropyKeywordCombinator


class TestCombinatorPlugin(unittest.TestCase):
    def setUp(self) -> None:
        self.plugin = EntropyKeywordCombinator()

    def test_positive_value(self):
        result = self.plugin.analyze_line("mock.tf", 'api_key = "wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMAAAKEY"', 5)
        self.assertEqual(1, len(result))
        secret = result.pop()
        self.assertEqual("Base64 High Entropy String", secret.type)
        self.assertEqual("c00f1a6e4b20aa64691d50781b810756d6254b8e", secret.secret_hash)

    def test_suspicious_keyword_no_secret(self):
        result = self.plugin.analyze_line("mock.json", "'wrong_one_time_password' = 'Du har tastet feil engangspassord'", 5)
        self.assertEqual(0, len(result))

    def test_suspicious_keyword_with_secret(self):
        result = self.plugin.analyze_line("mock.json", "'my_new_password':'F317a45xxmwov9bpgRhyuByXj2nxz7khS6yXQmfSaQCmwbTF1jpfgC56az4a'", 5)
        self.assertEqual(1, len(result))
        secret = result.pop()
        self.assertEqual("Base64 High Entropy String", secret.type)
        self.assertEqual("F317a45xxmwov9bpgRhyuByXj2nxz7khS6yXQmfSaQCmwbTF1jpfgC56az4a", secret.secret_value)

    def test_unquoted_secret(self):
        result = self.plugin.analyze_line("mock.yaml", 'export secret_access_key=wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMAAAKEY', 5)
        self.assertEqual(1, len(result))
        secret = result.pop()
        self.assertEqual("Base64 High Entropy String", secret.type)
        self.assertEqual("c00f1a6e4b20aa64691d50781b810756d6254b8e", secret.secret_hash)

    def test_negative_keyword_value(self):
        result = self.plugin.analyze_line("mock.tf", "wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMAAAKEY", 5)
        self.assertEqual(0, len(result))

    def test_negative_entropy_value(self):
        result = self.plugin.analyze_line("mock.tf", "api_key = var.api_key", 5)
        self.assertEqual(0, len(result))

    def test_popular_kubernetes_manifest_password(self):
        result = self.plugin.analyze_line("mock.yaml", 'pwd: "correcthorsebatterystaple"', 5)
        self.assertEqual(1, len(result))
        secret = result.pop()
        self.assertEqual("Base64 High Entropy String", secret.type)
        self.assertEqual("bfd3617727eab0e800e62a776c76381defbc4145", secret.secret_hash)

    def test_no_false_positive_py(self):
        # combinator plugin should ignore source code files
        result = self.plugin.analyze_line("main.py", 'api_key = "7T)G#dl5}c=T>kf$G3Bon^!R?kzF00"', 1)
        self.assertEqual(0, len(result))

    def test_no_false_positive_yml_1(self):
        test_file_path = Path(__file__).parent / "resources/cfn/secret-no-false-positive.yml"
        with open(file=str(test_file_path)) as f:
            for i, line in enumerate(f.readlines()):
                result = self.plugin.analyze_line("secret-no-false-positive.yml", line, i)
                self.assertEqual(0, len(result))

    def test_no_false_positive_yml_2(self):
        test_file_path = Path(__file__).parent / "resources/cfn/secret-no-false-positive2.yml"
        with open(file=str(test_file_path)) as f:
            for i, line in enumerate(f.readlines()):
                result = self.plugin.analyze_line("secret-no-false-positive2.yml", line, i)
                self.assertEqual(0, len(result))

    def test_no_false_positive_image_bytes(self):
        result = self.plugin.analyze_line("main.py", "'image/jpeg' : b'/9j/4AAQSkZJRgABAQAAAQABAAD/2wBDAAgGBgcGBQgHBwcJCQgKDBQNDAsLDBkSEw8UHRofHh0a\nHBwgJC4nICIsIxwcKDcpLDAxNDQ0Hyc5PTgyPC4zNDL/2wBDAQkJCQwLDBgNDRgyIRwhMjIyMjIy\nMjIyMjIyMjIyMjIyMjIyMjIyMjIyMjIyMjIyMjIyMjIyMjIyMjIyMjIyMjL/wAARCAACAAIDASIA\nAhEBAxEB/8QAHwAAAQUBAQEBAQEAAAAAAAAAAAECAwQFBgcICQoL/8QAtRAAAgEDAwIEAwUFBAQA\nAAF9AQIDAAQRBRIhMUEGE1FhByJxFDKBkaEII0KxwRVS0fAkM2JyggkKFhcYGRolJicoKSo0NTY3\nODk6Q0RFRkdISUpTVFVWV1hZWmNkZWZnaGlqc3R1dnd4eXqDhIWGh4iJipKTlJWWl5iZmqKjpKWm\np6ipqrKztLW2t7i5usLDxMXGx8jJytLT1NXW19jZ2uHi4+Tl5ufo6erx8vP09fb3+Pn6/8QAHwEA\nAwEBAQEBAQEBAQAAAAAAAAECAwQFBgcICQoL/8QAtREAAgECBAQDBAcFBAQAAQJ3AAECAxEEBSEx\nBhJBUQdhcRMiMoEIFEKRobHBCSMzUvAVYnLRChYkNOEl8RcYGRomJygpKjU2Nzg5OkNERUZHSElK\nU1RVVldYWVpjZGVmZ2hpanN0dXZ3eHl6goOEhYaHiImKkpOUlZaXmJmaoqOkpaanqKmqsrO0tba3\nuLm6wsPExcbHyMnK0tPU1dbX2Nna4uPk5ebn6Onq8vP09fb3+Pn6/9oADAMBAAIRAxEAPwD5/ooo\noAoo2Qoo'", 1)
        self.assertEqual(0, len(result))

    def test_no_false_positive_token(self):
        result = self.plugin.analyze_line("main.go", 'fmt.Sprintf("https://%s:%s@", token, token)', 1)
        self.assertEqual(0, len(result))

    def test_secret_value_in_keyword(self):
        result = self.plugin.analyze_line("mock.tf", 'export AWS_SECRET_ACCESS_KEY=h4t2TJheVRR8em5VdNCjrSJdQ+p7OHl33SxrZoUi', 1)
        self.assertEqual(1, len(result))

    def test_k8s_secret_name(self):
        # given
        test_file_path = Path(__file__).parent / "resources/k8s/secret-name.yaml"

        # when
        with open(file=str(test_file_path)) as f:
            for i, line in enumerate(f.readlines()):
                result = self.plugin.analyze_line("secret-no-false-positive2.yml", line, i)

                # then
                self.assertEqual(0, len(result))
